package cmdbuild.collection;

import java.io.Serializable;

public class AttributeBean  implements Serializable,AttributeInterface{
	
	static final long serialVersionUID = 1L;
	private ClassBean classBean = new ClassBean();
    private String name;
    private int index;
    private String description;
    private String SQLType; // type in database
    private int type;    	// type used in drop down window (select) in the form
    private boolean dateExpire; // when a type "date" expires
    private String lookup=""; // initialization required
    private String reference=""; // initialization required
    private String referenceDomain="";
    private String referenceType=""; // type of cancellation
    private boolean referenceDirect;
    private boolean showMenu; // derived from master/detail domains
    private String mandatory="";
    private String defaultValue=""; // initialization required
    private String baseDSP; // Base Display / show this when seeing cards / values: "true" OR "false" OR ""
    private String status="active";
    private String mode;
    private String comment; // necessary for changing comments on columns (Schema)
    private int DBIndex;
    // other
    private int length; 	// used to store varchar length
    private String extraInput;

	public AttributeBean() {
		super();
	}
	
	public AttributeBean(String name, String type, int type2, int length, String mandatory, String description) {
		super();
		// TODO Auto-generated constructor stub
		this.name = name;
		this.SQLType = type;
		this.type = type2;
		this.length = length;
		this.mandatory = mandatory;
		this.description = description;
	}

	public AttributeBean(String name, String type, int type2, int length, String mandatory, String description, String baseDSP) {
		super();
		// TODO Auto-generated constructor stub
		this.name = name;
		this.SQLType = type;
		this.type = type2;
		this.length = length;
		this.mandatory = mandatory;
		this.description = description;
		this.baseDSP = baseDSP;
	}
	
	public static AttributeBean getCopyOf(AttributeBean ab) {
		AttributeBean copy = new AttributeBean();
	    ClassBean cb = ab.getClassBean();
	    copy.setClassBean(ClassBean.getCopyOf(cb));
	    copy.setName(ab.getName());
	    copy.setSQLType(ab.getSQLType());
	    copy.setType(ab.getType());
	    copy.setLength(ab.getLength());
	    copy.setMandatory(ab.getMandatory());
	    copy.setDefaultValue(ab.getDefaultValue());
	    copy.setMode(ab.getMode());
	    copy.setDescription(ab.getDescription());
	    copy.setLookup(ab.getLookup());
	    copy.setReference(ab.getReference());
	    copy.setReferenceDomain(ab.getReferenceDomain());
	    copy.setReferenceType(ab.getReferenceType());
	    copy.setReferenceDirect(ab.isReferenceDirect());
	    copy.setShowMenu(ab.isShowMenu());
	    copy.setDateExpire(ab.isDateExpire());
	    copy.setStatus(ab.getStatus());
	    copy.setComment(ab.getComment());
	    copy.setIndex(ab.getIndex());
	    copy.setDBIndex(ab.getDBIndex());
	    copy.setBaseDSP(ab.getBaseDSP());
	    copy.setExtraInput(ab.getExtraInput());
		return copy;
	}

	public String javaToDb(int type) {
		String stype = "";
		switch (type) {
        	case AttributeBean.BOOLEAN	:  stype ="bool"; break;
        	case AttributeBean.INTEGER	:  stype ="int4"; break;
        	case AttributeBean.DECIMAL	:  stype ="numeric"; break;
        	case AttributeBean.DOUBLE	:  stype ="float8"; break;
        	case AttributeBean.DATE		:  stype ="date"; break;
        	case AttributeBean.TIMESTAMP:  stype ="timestamp"; break;
        	case AttributeBean.LOOKUP	:  stype ="int4"; break;
        	case AttributeBean.REFERENCE:  stype ="int4"; break;
        	case AttributeBean.TEXT		:  stype ="text"; break;
        	case AttributeBean.STRING	:  stype ="varchar"; break;
        	default						:  stype ="varchar"; break;
		}
		return stype;
	}
	
	/***********************/
	/* GETTERS AND SETTERS */
	/***********************/

	public String getDefaultValue() {
		if (defaultValue==null){
			defaultValue = "";
		}
		return defaultValue;
	}
	public void setDefaultValue(String default1) {
		int s,e;
		if (default1!=null) {
			if (default1.indexOf("'") != -1) { // type lookup is modified when calling setALookup
				s = default1.indexOf("'");  // 'ciao'::character varying
				e = default1.lastIndexOf("'");
				if ((s!=-1) && (e!=-1)){
					defaultValue = default1.substring(s+1,e);
				}
			} else {
				defaultValue = default1;
			}
		} else { // is null
			defaultValue = default1;
		}
	}
	public String getDescription() {
		return description;
	}
	public void setDescription(String description) {
		this.description = description;
	}
	public String getLookup() {
		return lookup;
	}
	public void setLookup(String lookup) { 
		if ( (lookup==null) || lookup.trim().equals("") ){
			this.lookup = lookup;
		} else {
			this.lookup = lookup;
			this.type = AttributeBean.LOOKUP;	// now aType is lookup, before was a varchar
		}
	}
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	public int getIndex() {
		return index;
	}
	public void setIndex(int index) {
		this.index = index;
	}
	public int getDBIndex() {
		return DBIndex;
	}
	public void setDBIndex(int index) {
		DBIndex = index;
	}
	public String getMandatory() {
		if (type==AttributeBean.REFERENCE){
			this.mandatory = "Y";
		}	
		return mandatory;
	}
	public void setMandatory(String mandatory) {
		if (type==AttributeBean.REFERENCE){
			this.mandatory = "Y";
		} else if (mandatory!=null){ 
			if (mandatory.equals("")){
				this.mandatory= "N";
			} else if (mandatory.indexOf("NOT NULL")!=-1){
				this.mandatory = "Y";
			} else if (mandatory.equals("Y")){
				this.mandatory = "Y";
			} else {
				this.mandatory= "N";
			}
		} else {
			this.mandatory = mandatory;
		}
	}
	public String getStatus() {
		return status;
	}
	public void setStatus(String status) {
		if ((status==null) || (status.equals(""))){
			this.status = "active";
		} else {
			this.status = status;
		}
	}
	public String getSQLType() {
		return SQLType;
	}
	public void setSQLType(String type) {
		SQLType = type;
	}
	public int getType() {
		return type;
	}
	public void setType(int type) { 
		this.type = type;
	}
	public void setType(String SQLtype) { //transform SQLtype in aType
		int s,e;
		//System.out.println("type in AttributeBean="+type);
		if (SQLtype!=null && SQLtype.indexOf("numeric") != -1) {
			this.type = AttributeBean.DECIMAL;
			s = SQLtype.indexOf("(");
			e = SQLtype.indexOf(")");
			if ((s!=-1) && (e!=-1)){
				extraInput = SQLtype.substring(s,e);
			}
		} else if (SQLtype!=null && SQLtype.indexOf("varchar") != -1) { // type lookup is modified when calling setALookup
			this.type = AttributeBean.STRING;
			s = SQLtype.indexOf("(");
			e = SQLtype.indexOf(")");
			if ((s!=-1) && (e!=-1)){
				extraInput = SQLtype.substring(s,e);
				this.length= Integer.parseInt(extraInput);
			}
		} else if (SQLtype!=null && SQLtype.indexOf("char") != -1){ // bpchar
			this.type = AttributeBean.STRING;
		} else if (SQLtype!=null && SQLtype.indexOf("int") != -1) {
			this.type = AttributeBean.INTEGER;
		} else if (SQLtype!=null && SQLtype.indexOf("float") != -1) {
			this.type = AttributeBean.DOUBLE;
		} else if (SQLtype!=null && SQLtype.indexOf("text") != -1) {
			this.type = AttributeBean.TEXT;
		} else if (SQLtype!=null && SQLtype.indexOf("date") != -1) {
			this.type = AttributeBean.DATE;
		} else if (SQLtype!=null && SQLtype.indexOf("time") != -1) {
			this.type = AttributeBean.TIMESTAMP;
		} else if (SQLtype!=null && SQLtype.indexOf("bool") != -1) {
			this.type = AttributeBean.BOOLEAN;
		} else {
			this.type = AttributeBean.STRING;
		}
	}
	public int getLength() {
		return length;
	}
	public void setLength(int length) {
		this.length = length;
	}
	public String getExtraInput() {
		return extraInput;
	}
	public void setExtraInput(String extraInput) {
		this.extraInput = extraInput;
	}
	public String getMode() {
		return mode;
	}
	public void setMode(String mode) {
		this.mode = mode;
	}
	public String getBaseDSP() {
		return baseDSP;
	}
	public void setBaseDSP(String baseDSP) {
		if (baseDSP==null){
			this.baseDSP = "";
		} else {
			this.baseDSP = baseDSP;
		}
	}
	public String getComment() {
		return comment;
	}
	public void setComment(String comment) {
		this.comment = comment;
	}
	public ClassBean getClassBean() {
		return classBean;
	}
	public void setClassBean(ClassBean classBean) {
		this.classBean = classBean;
	}
	public boolean isDateExpire() {
		return dateExpire;
	}
	public void setDateExpire(boolean dateExpire) {
		this.dateExpire = dateExpire;
	}
	public String getReference() {
		if (reference==null){
			reference = "";
		}
		return reference;
	}
	public void setReference(String reference) {
		this.reference = reference;
	}
	public boolean isReferenceDirect() {
		return referenceDirect;
	}
	public void setReferenceDirect(boolean referenceDirect) {
		this.referenceDirect = referenceDirect;
	}
	public String getReferenceDomain() {
		if (referenceDomain==null){
			referenceDomain = "";
		}
		return referenceDomain;
	}
	public void setReferenceDomain(String referenceDomain) {
		this.referenceDomain = referenceDomain;
		if ( referenceDomain!=null && referenceDomain.length()>0 ){
			this.type = AttributeBean.REFERENCE;	// now aType is reference, before was a varchar
		}
	}
	public String getReferenceType() {
		return referenceType;
	}
	public void setReferenceType(String referenceType) {
		this.referenceType = referenceType;
	}
	public boolean isShowMenu() {
		return showMenu;
	}
	public void setShowMenu(boolean showMenu) {
		this.showMenu = showMenu;
	}

} // end of class
