package cmdbuild.elements;

import cmdbuild.db.DBService;
import cmdbuild.collection.AttributeBean;
import cmdbuild.collection.DomainBean;
import cmdbuild.collection.HistoryBean;
import cmdbuild.collection.RelationBean;

import cmdbuild.collection.IdBean;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.Enumeration;
import java.util.Vector;

public class CMDBmap extends CMDBtemplate{

	String CREATE_DOMAIN = "{ ?= call createDomain(?,?,?,?,?,?,?,?) }"; // Name,class1,class2,directdescrition,inversedescription,masterdetail,cardinality,status
	String MODIFY_DOMAIN = "{ ?= call modifyDomain(?,?,?,?,?,?,?,?,?) }"; // Name,newName,class1,class2,directdescrition,inversedescription,masterdetail,cardinality,status
	String DELETE_DOMAIN = "{ ?= call deleteDomain(?) }"; // Name
	
	public CMDBmap(DBService db) {
		super(db);
	}

	public boolean isEmpy(String name) throws SQLException{
		// adding Map_ beacause isempty is used by classes and domains too
		if (name.indexOf("Map")==-1){
			name = "Map_"+name;
		}
		boolean empty = false;
		String query = "SELECT isempty('"+name+"');";
		ResultSet rs = null;
		PreparedStatement stm = null;
		DEBUG.println("Query ISEMPTY="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				if ( rs.next() ) { // while ( rs.next() ) {
					empty = rs.getBoolean(1);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally { // added**
			DBService.close(rs, stm, con);
		}
		return empty;
	}

	// **********
	// **domain**
	// **********
	public boolean createDomain(DomainBean bean) throws SQLException {
		boolean ok = false;
		CallableStatement stm = null;
		DEBUG.println("createdomain("+bean.getName()+","+bean.getStartClass()+","+bean.getEndClass()+","+bean.getDirectDescription()+","+bean.getInverseDescription()+","+bean.isMasterDetail()+","+bean.getCardinality()+","+bean.getStatus()+")");
		Connection con = db.getConnection();
		try {
			stm = con.prepareCall(CREATE_DOMAIN);
			stm.registerOutParameter(1,Types.BOOLEAN);
			stm.setString(2,bean.getName());
			stm.setString(3,bean.getStartClass());
			stm.setString(4,bean.getEndClass());
			stm.setString(5,bean.getDirectDescription());
			stm.setString(6,bean.getInverseDescription());
			stm.setString(7,Boolean.toString(bean.isMasterDetail()));
			stm.setString(8,bean.getCardinality());
			stm.setString(9,bean.getStatus());
			stm.execute();
			ok = true;
		} catch (SQLException ex) {
			throw ex;
		} finally {
			DBService.close(null, stm, con);
		}
		return ok;
	}
	public boolean modifyDomain(String newname, DomainBean bean) throws SQLException {
		boolean ok = false;
		CallableStatement stm = null;
		DEBUG.println("modifydomain("+bean.getName()+","+bean.getStartClass()+","+bean.getEndClass()+","+bean.getDirectDescription()+","+bean.getInverseDescription()+","+bean.isMasterDetail()+","+bean.getCardinality()+","+bean.getStatus()+")");
		Connection con = db.getConnection();
		try {
			stm = con.prepareCall(MODIFY_DOMAIN);
			stm.registerOutParameter(1,Types.BOOLEAN);
			stm.setString(2,bean.getName());
			stm.setString(3,newname);
			stm.setString(4,bean.getStartClass());
			stm.setString(5,bean.getEndClass());
			stm.setString(6,bean.getDirectDescription());
			stm.setString(7,bean.getInverseDescription());
			stm.setString(8,Boolean.toString(bean.isMasterDetail()));
			stm.setString(9,bean.getCardinality());
			stm.setString(10,bean.getStatus());
			stm.execute();
			ok = true;
		} catch (SQLException ex) {
			throw ex;
		} finally {
			DBService.close(null, stm, con);
		}
		return ok;
	}
	public boolean deleteDomain(String name) throws SQLException{
		boolean ok = false;
		CallableStatement stm = null;
		DEBUG.println("deletedomain("+name+")");
		Connection con = db.getConnection();
		try {
			stm = con.prepareCall(DELETE_DOMAIN);
			stm.registerOutParameter(1,Types.BOOLEAN);
			stm.setString(2,name);
			stm.execute();
			ok = true;
		} catch (SQLException ex) {
			throw ex;
		} finally {
			DBService.close(null, stm, con);
		}
		return ok;
	}
	public  Vector selectDomainByClass(String classname) throws SQLException {
		Vector domainBeanVector = new Vector();
		ResultSet rs = null;
		PreparedStatement stm = null;
		DomainBean obj = null;
		String query = "SELECT domainid,domainname,domaindirectdescription,domaininversedescription,domainclass1,domainclass2,domainmasterdetail,domaincardinality,domainstatus FROM cmdbdomaincatalog WHERE domainmode='read' AND \"cmdbdomaincatalog\".\"domainclass1\"='"+classname+"' OR \"cmdbdomaincatalog\".\"domainclass2\"='"+classname+"'";
		DEBUG.println("Query SELECTDOMAINBYCLASS="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				while ( rs.next() ) {
					obj = setDomainObject(rs);
					domainBeanVector.add(obj);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return domainBeanVector;
	}
	public  DomainBean selectDomainByName(String name) throws SQLException {
		ResultSet rs = null;
		PreparedStatement stm = null;
		DomainBean obj = null;
		if (name.indexOf("Map_")==-1){
			name = "Map_"+name;
		}
		String query = "SELECT domainid,domainname,domaindirectdescription,domaininversedescription,domainclass1,domainclass2,domainmasterdetail,domaincardinality,domainstatus FROM cmdbdomaincatalog WHERE domainmode='read' AND \"cmdbdomaincatalog\".\"domainname\"='"+name+"'";
		DEBUG.println("Query SELECTDOMAINBYNAME="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				if ( rs.next() ) {
					obj = setDomainObject(rs);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return obj;
	}
	public Vector selectAllDomain() throws SQLException {
		Vector vect = new Vector();
		ResultSet rs = null;
		PreparedStatement stm = null;
		DomainBean obj = null;
		String query = "SELECT domainid,domainname,domaindirectdescription,domaininversedescription,domainclass1,domainclass2,domainmasterdetail,domaincardinality,domainstatus FROM cmdbdomaincatalog WHERE domainmode='read'";
		DEBUG.println("Query SELECTALLDOMAIN="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				while( rs.next() ) {
					obj = setDomainObject(rs);
					vect.add(obj);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return vect;
	}
	public  Vector selectDomainInvolvedReferences(String domain) throws SQLException {
		Vector attributeBeanVector = new Vector();
		ResultSet rs = null;
		PreparedStatement stm = null;
		AttributeBean obj = null;
		String query="";
		query = "SELECT classid,classname,classcomment,classidcategory,classcategory,classdescription,classmode,issuperclass,classmanager,classstatus,attributename,attributetype,attributelength,attributenull,attributedefault,attributemode,attributedescription,attributelookup,attributereference,attributereferencedomain,attributereferencetype,attributereferencedirect,attributedateexpire,attributebasedsp,attributestatus,attributeindex,attributedbindex,attributecomment FROM cmdbclasscatalog WHERE attributereferencedomain='"+domain+"' ORDER BY attributeindex,attributedbindex;";	
		DEBUG.println("Query SELECTDOMAININVOLVEDREFERENCES="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				while ( rs.next() ) {
					obj = setAttributeObject(rs);
					attributeBeanVector.add(obj);
				}
			}	
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return attributeBeanVector;
	}

	// **********
	// **relation
	// **********
	public Vector existsRelation(DomainBean dm, boolean direct, Vector col) throws SQLException {
		Vector result = new Vector();
		ResultSet rs = null;
		PreparedStatement stm = null;
		Enumeration en = col.elements();
		IdBean ib = null;
		String query = "";
		String domainName = dm.getName(); 
		if (domainName.indexOf("Map_")==-1){
			domainName = "Map_"+domainName;
		}
		// select count(*) from "Map_assegnatario" where ("IdClass1"=744121 AND "IdObj1"=0)
		while (en.hasMoreElements()){
			ib = (IdBean) en.nextElement();
			if (query.length()>0){
				if (direct){
					query += " OR (\"IdClass1\"="+ib.getClassId()+" AND \"IdObj1\"="+ib.getId()+")";
				} else {
					query += " OR (\"IdClass2\"="+ib.getClassId()+" AND \"IdObj2\"="+ib.getId()+")";
				}
			} else { // first time
				if (direct){
					query = "SELECT \"IdClass1\",\"IdObj1\",COUNT(\"IdObj1\") FROM \""+domainName+"\" WHERE (\"IdClass1\"="+ib.getClassId()+" AND \"IdObj1\"="+ib.getId()+")";
				} else {
					query = "SELECT \"IdClass2\",\"IdObj2\",COUNT(\"IdObj2\") FROM \""+domainName+"\" WHERE (\"IdClass2\"="+ib.getClassId()+" AND \"IdObj2\"="+ib.getId()+")";
				}
				
			}
		}
		if (direct){
			query += " AND \"Status\"='A' GROUP BY  \"IdClass1\",\"IdObj1\";";
		} else {
			query += " AND \"Status\"='A' GROUP BY  \"IdClass2\",\"IdObj2\";";
		}
		// SELECT "IdObj2",COUNT("IdObj2") from "Map_fornitore" where (("IdClass2"=945182 AND "IdObj2"=4165) OR ("IdClass2"=945182 AND "IdObj2"=4166)) AND "Status"='A' GROUP BY  "IdObj2";
		// adding double brackets
		String q1 = query.substring(0,query.indexOf("WHERE")+6);
		String q2 = query.substring(query.indexOf("WHERE")+6,query.lastIndexOf(")"));
		String q3 = query.substring(query.lastIndexOf(")"),query.length());
		query = q1+"("+q2+")"+q3;
		DEBUG.println("Query EXISTSRELATION="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				while( rs.next() ) {
					ib = new IdBean();
					ib.setClassId(rs.getInt(1));
					ib.setId(rs.getInt(2));
					ib.setName(rs.getString(3)); // count
					result.add(ib);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return result;
	}
	public  boolean createRelation(String domainName, int domainId, int c1, int o1, int c2, int o2, String user) throws SQLException {
		int r=0;
		boolean result;
		Statement stm = null;
		ResultSet rs = null;
		if (domainName.indexOf("Map_")==-1){
			domainName = "Map_"+domainName;
		}
		String query="";
		if (user!=null){
			query = "INSERT INTO \""+domainName+"\" (\"IdDomain\",\"IdClass1\",\"IdObj1\",\"IdClass2\",\"IdObj2\",\"Status\",\"User\") VALUES ("+domainId+","+c1+","+o1+","+c2+","+o2+",'A','"+user+"')";
		} else {
			query = "INSERT INTO \""+domainName+"\" (\"IdDomain\",\"IdClass1\",\"IdObj1\",\"IdClass2\",\"IdObj2\",\"Status\") VALUES ("+domainId+","+c1+","+o1+","+c2+","+o2+",'A')";	
		}
		DEBUG.println("Query CREATERELATION="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.createStatement();
				r= stm.executeUpdate(query) ;
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		if (r==1) {
			result = true;
		} else {
			result = false;
		}
		return result;
	}
	public  boolean createRelationBatch(DomainBean dm, boolean direct, Vector co1, Vector co2, String user) throws SQLException {
		boolean result=false;
		ResultSet rs = null;
		String domainName = dm.getName();
		if (domainName.indexOf("Map_")==-1){
			domainName = "Map_"+domainName;
		}
		String query = "";
		Connection con = db.getConnection();
		Statement stm = con.createStatement();
		int newc = ((IdBean)co2.elementAt(0)).getClassId();
		int newo = ((IdBean)co2.elementAt(0)).getId();
		int curc = 0;
		int curo = 0;
		DEBUG.println("Query CREATERELATIONBATCH co1="+co1.size()+" co2="+co2.size());
		Enumeration en = co1.elements();
		IdBean idb = null;
		if (direct){
			while (en.hasMoreElements()){
				idb = ((IdBean)en.nextElement());
				curc = idb.getClassId();
				curo = idb.getId();
				query = "INSERT INTO \""+domainName+"\" (\"IdDomain\",\"IdClass1\",\"IdObj1\",\"IdClass2\",\"IdObj2\",\"Status\",\"User\") VALUES ("+dm.getId()+","+curc+","+curo+","+newc+","+newo+",'A','"+user+"') ; ";
				DEBUG.println("Query CREATERELATIONBATCH="+query);
				stm.addBatch(query);
			}
		} else {
			while (en.hasMoreElements()){
				idb = ((IdBean)en.nextElement());
				curc = idb.getClassId();
				curo = idb.getId();
				query = "INSERT INTO \""+domainName+"\" (\"IdDomain\",\"IdClass1\",\"IdObj1\",\"IdClass2\",\"IdObj2\",\"Status\",\"User\") VALUES ("+dm.getId()+","+newc+","+newo+","+curc+","+curo+",'A','"+user+"') ; ";
				DEBUG.println("Query CREATERELATIONBATCH="+query);
				stm.addBatch(query);
			}
		}
		int tots[] = null;
		int tot = 0;
		try {
			if ( con != null ) {
				tots = stm.executeBatch();
				result = true;
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		if (tots!=null){
			for (int i=0;i<tots.length;i++){
				tot = tot+tots[i];
			}
		}
		DEBUG.println(tot+" lines modified");
		return result;
	}
	public  boolean modifyRelation(String domainName, int domainId, boolean direct, int c1, int o1, int c2, int o2, int newc, int newo, String user) throws SQLException {
		int r=0;
		boolean result;
		Statement stm = null;
		ResultSet rs = null;
		if (domainName.indexOf("Map_")==-1){
			domainName = "Map_"+domainName;
		}
		DEBUG.println("direct is "+direct);
		String query = "";
		if (direct) {
			if (user!=null){
				query = "UPDATE \""+domainName+"\" SET \"IdClass2\"='"+newc+"' , \"IdObj2\"='"+newo+"' ,\"User\"='"+user+"' WHERE \"IdDomain\"='"+domainId+"' AND \"IdClass1\"='"+c1+"' AND \"IdObj1\"='"+o1+"' AND \"IdClass2\"='"+c2+"' AND \"IdObj2\"='"+o2+"'";
			} else {
				query = "UPDATE \""+domainName+"\" SET \"IdClass2\"='"+newc+"' , \"IdObj2\"='"+newo+"' WHERE \"IdDomain\"='"+domainId+"' AND \"IdClass1\"='"+c1+"' AND \"IdObj1\"='"+o1+"' AND \"IdClass2\"='"+c2+"' AND \"IdObj2\"='"+o2+"'";	
			}			
		} else { // !direct = inverseDescription
			if (user!=null){ 
				query = "UPDATE \""+domainName+"\" SET \"IdClass1\"='"+newc+"' , \"IdObj1\"='"+newo+"' ,\"User\"='"+user+"' WHERE \"IdDomain\"='"+domainId+"' AND \"IdClass1\"='"+c2+"' AND \"IdObj1\"='"+o2+"' AND \"IdClass2\"='"+c1+"' AND \"IdObj2\"='"+o1+"'";	
			} else {
				query = "UPDATE \""+domainName+"\" SET \"IdClass1\"='"+newc+"' , \"IdObj1\"='"+newo+"' WHERE \"IdDomain\"='"+domainId+"' AND \"IdClass1\"='"+c2+"' AND \"IdObj1\"='"+o2+"' AND \"IdClass2\"='"+c1+"' AND \"IdObj2\"='"+o1+"'";
			}
		}
		DEBUG.println("Query MODIFYRELATION="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.createStatement();
				r= stm.executeUpdate(query);
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		if (r>0) {
			result = true;
		} else {
			result = false;
		}
		return result;
	}
	public  boolean modifyRelationBatch(DomainBean dm, boolean direct, Vector co1, Vector co2, String user) throws SQLException {
		boolean result=false;
		ResultSet rs = null;
		String domainName = dm.getName();
		if (domainName.indexOf("Map_")==-1){
			domainName = "Map_"+domainName;
		}
		String query = "";
		Connection con = db.getConnection();
		Statement stm = con.createStatement();
		int newc = ((IdBean)co2.elementAt(0)).getClassId();
		int newo = ((IdBean)co2.elementAt(0)).getId();
		int curc = 0;
		int curo = 0;
		DEBUG.println("Query MODIFYRELATIONBATCH co1="+co1.size()+" co2="+co2.size());
		Enumeration en = co1.elements();
		IdBean idb = null;
		if (direct){
			while (en.hasMoreElements()){
				idb = ((IdBean)en.nextElement());
				curc = idb.getClassId();
				curo = idb.getId();
				query = "UPDATE \""+domainName+"\" SET \"IdClass2\"='"+newc+"' , \"IdObj2\"='"+newo+"' ,\"User\"='"+user+"' WHERE \"IdDomain\"='"+dm.getId()+"' AND \"IdClass1\"='"+curc+"' AND \"IdObj1\"='"+curo+"' AND \"Status\"='A'; ";
				DEBUG.println("Query MODIFYRELATIONBATCH="+query);
				stm.addBatch(query);
			}
		} else {
			while (en.hasMoreElements()){
				idb = ((IdBean)en.nextElement());
				curc = idb.getClassId();
				curo = idb.getId();
				query = "UPDATE \""+domainName+"\" SET \"IdClass1\"='"+newc+"' , \"IdObj1\"='"+newo+"' ,\"User\"='"+user+"' WHERE \"IdDomain\"='"+dm.getId()+"' AND \"IdClass2\"='"+curc+"' AND \"IdObj2\"='"+curo+"' AND \"Status\"='A'; ";
				DEBUG.println("Query MODIFYRELATIONBATCH="+query);
				stm.addBatch(query);
			}
		}
		int tots[] = null;
		int tot = 0;
		try {
			if ( con != null ) {
				tots = stm.executeBatch();
				result = true;
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		if (tots!=null){
			for (int i=0;i<tots.length;i++){
				tot = tot+tots[i];
			}
		}
		DEBUG.println(tot+" lines modified");
		return result;
	}
	public  boolean deleteRelation(String domainName, int domainId, boolean direct, int c1, int o1, int c2, int o2, String user) throws SQLException {
		int r=0;
		boolean result;
		Statement stm = null;
		ResultSet rs = null;
		if (domainName.indexOf("Map_")==-1){
			domainName = "Map_"+domainName;
		}
		String query = "";
		if (direct) {
			query = "DELETE FROM \""+domainName+"\" WHERE \"IdDomain\"='"+domainId+"' AND \"IdClass1\"='"+c1+"' AND \"IdObj1\"='"+o1+"' AND \"IdClass2\"='"+c2+"' AND \"IdObj2\"='"+o2+"'";
		} else { // !direct = inverseDescription
			query = "DELETE FROM \""+domainName+"\" WHERE \"IdDomain\"='"+domainId+"' AND \"IdClass1\"='"+c2+"' AND \"IdObj1\"='"+o2+"' AND \"IdClass2\"='"+c1+"' AND \"IdObj2\"='"+o1+"'";
		}
		DEBUG.println("Query DELETERELATION="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.createStatement();
				r= stm.executeUpdate(query);
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		if (r>0) {
			result = true;
		} else {
			result = false;
		}
		return result;
	}
	// ************
	// **relation**
	// ************
	public Vector selectRelationTips(String couple_class_obj) throws SQLException {
		//  couple_class_obj is like (idclass1=122 AND idobj1=12) OR (idclass1=122 AND idobj1=13) ......
		String query;
		if (couple_class_obj==null){
			couple_class_obj="";
		} else if (couple_class_obj.equals("")){
			
		} else {
			couple_class_obj=" WHERE "+couple_class_obj;
		}
		query = "SELECT idclass1,idobj1,domaindescription,domainmasterdetail,classdescription,fieldcode,fielddescription FROM relationlist";
		query += couple_class_obj;
		if (!couple_class_obj.equals("")){
			query+=" AND status='A' AND domainmasterdetail<>'true'"; 
		} else {
			query+=" WHERE status='A' AND domainmasterdetail<>'true'";
		}
		query += " ORDER BY \"idclass1\",\"idobj1\" ";
		Vector relationTipsVector = new Vector();
		DEBUG.println("Query SELECTRELATIONTIPS="+query);
		ResultSet rs = null;
		PreparedStatement stm = null;
		RelationBean rb = null;
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				while ( rs.next() ) {
					rb = new RelationBean();
					rb.setClassId(rs.getInt(1));
					rb.setObjId(rs.getInt(2));
					rb.setDomainDescription(rs.getString(3));
					// domainmasterdetail 4
					rb.setDomainMasterDetail(false);
					rb.setClassDescription(rs.getString(5));
					rb.setCode(rs.getString(6));
					rb.setDescription(rs.getString(7));
					relationTipsVector.add(rb);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return relationTipsVector;
	}
	public  Vector selectRelations(int IdClass, int rowId, int limit, int offset, int domainType) throws SQLException {
		Vector relationVector = new Vector();
		RelationBean bean = null;
		ResultSet rs = null;
		PreparedStatement stm = null;
		// retrieve information of the other object
		String query = "";
		switch (domainType) {
			case DomainBean.ALL: query = "SELECT class1,class2,fieldcode,fielddescription,realname,iddomain,idclass1,idobj1,idclass2,idobj2,status,domaindescription,domainmasterdetail,domaincardinality,classdescription,direct FROM relationlist WHERE idclass1="+IdClass+" AND idobj1="+rowId+" AND status='A' ";
			break;
			case DomainBean.NORMAL: query = "SELECT class1,class2,fieldcode,fielddescription,realname,iddomain,idclass1,idobj1,idclass2,idobj2,status,domaindescription,domainmasterdetail,domaincardinality,classdescription,direct FROM relationlist WHERE idclass1="+IdClass+" AND idobj1="+rowId+" AND status='A' AND domainmasterdetail<>'true' ";
			break;
			case DomainBean.MASTERDETAIL: query = "SELECT class1,class2,fieldcode,fielddescription,realname,iddomain,idclass1,idobj1,idclass2,idobj2,status,domaindescription,domainmasterdetail,domaincardinality,classdescription,direct FROM relationlist WHERE idclass1="+IdClass+" AND idobj1="+rowId+" AND status='A' AND domainmasterdetail='true' ";
			break;
		}
		if (limit!=0){
			query+= " LIMIT "+limit;
		}
		if (offset!=0){
			query+= " OFFSET "+offset;
		}
		DEBUG.println("Query SELECTRELATIONS="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				while ( rs.next() ) { // while ( rs.next() ) {
					//bean.setClassName(rs.getString(1));
					bean = setRelationObject(rs);
					relationVector.add(bean);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return relationVector;
	}
	public Vector selectHistoryRelations(int IdClass, int rowId, int domainType) throws SQLException {
		Vector relationVector = new Vector();
		HistoryBean his = null;
		ResultSet rs = null;
		PreparedStatement stm = null;
		// retrieve information of the other object
		String query = "";
		switch (domainType) {
			case DomainBean.ALL: query = "SELECT idclass2,idobj2,domaindescription,domainmasterdetail,fieldcode,fielddescription,username,begindate,enddate FROM relationhistorylist WHERE idclass1="+IdClass+" AND idobj1="+rowId+" AND status<>'A'; ";
			break;
			case DomainBean.NORMAL: query = "SELECT idclass2,idobj2,domaindescription,domainmasterdetail,fieldcode,fielddescription,username,begindate,enddate FROM relationhistorylist WHERE idclass1="+IdClass+" AND idobj1="+rowId+" AND status<>'A' AND domainmasterdetail<>'true'; ";
			break;
			case DomainBean.MASTERDETAIL: query = "SELECT idclass2,idobj2,domaindescription,domainmasterdetail,fieldcode,fielddescription,username,begindate,enddate FROM relationhistorylist WHERE idclass1="+IdClass+" AND idobj1="+rowId+" AND status<>'A' AND domainmasterdetail='true'; ";
			break;
		}
		DEBUG.println("Query SELECTHISTORYRELATION="+query);
		Connection con = db.getConnection();
		String str = "";
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				while ( rs.next() ) { // while ( rs.next() ) {
					//bean.setClassName(rs.getString(1));
					his = new HistoryBean();
					his.setClassId(rs.getInt(1));
					his.setObjId(rs.getInt(2));
					his.setRelationName(rs.getString(3));
					str = rs.getString(4);
					if (str.length()>0 && str.equals("true")){
						his.setRelationMasterDetail(true);
					} else {
						his.setRelationMasterDetail(false);
					}
					his.setRelatedObjectCode(rs.getString(5));
					his.setRelatedObjectDescription(rs.getString(6));
					his.setUser(rs.getString(7));
					his.setBeginDate(rs.getString(8));
					his.setEndDate(rs.getString(9));
					his.setAttribute(false);
					relationVector.add(his);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return relationVector;
	}


	/**************************/
	/* USEFUL SETTING METHOTS */
	/**************************/
	
	private DomainBean setDomainObject(ResultSet rs) {
		DomainBean bean = new DomainBean();
		String nam ="";
		try {
			bean.setId(rs.getInt(1));
			nam = rs.getString(2);
			if (nam.indexOf("Map_")!=-1){
				nam = nam.substring(4);
			}
			bean.setName(nam);
			bean.setDirectDescription(rs.getString(3));
			bean.setInverseDescription(rs.getString(4));
			bean.setStartClass(rs.getString(5));
			bean.setEndClass(rs.getString(6));
			bean.setMasterDetail(rs.getBoolean(7));
			bean.setCardinality(rs.getString(8));
			bean.setStatus(rs.getString(9));
		} catch (SQLException e) {
			e.printStackTrace();
		}		
		return bean;
	}

	private RelationBean setRelationObject(ResultSet rs) {
		RelationBean bean = new RelationBean();	
		String str = "";
		try {
			bean.setClassName(rs.getString(2));
			bean.setCode(rs.getString(3));
			bean.setDescription(rs.getString(4));
			bean.setDomainRealName(rs.getString(5));
			bean.setDomainId(rs.getInt(6));
			bean.setClassId(rs.getInt(9));
			bean.setObjId(rs.getInt(10));
			bean.setStatus(rs.getString(11));
			bean.setDomainDescription(rs.getString(12));
			str = rs.getString(13);
			if (str.length()>0 && str.equals("true")){
				bean.setDomainMasterDetail(true);
			} else {
				bean.setDomainMasterDetail(false);
			}
			// cardinality 14
			bean.setClassDescription(rs.getString(15));
			bean.setDirect(rs.getBoolean(16));
		} catch (SQLException e) {
			e.printStackTrace();
		}		
		return bean;
	}
	
	private AttributeBean setAttributeObject(ResultSet rs) {
		AttributeBean bean = new AttributeBean();		
		try {
			// class info
			bean.getClassBean().setId(rs.getInt(1));
			bean.getClassBean().setName(rs.getString(2));
			bean.getClassBean().setComment(rs.getString(3));
			bean.getClassBean().setIdCategory(rs.getInt(4));
			bean.getClassBean().setCategory(rs.getString(5));
			bean.getClassBean().setDescription(rs.getString(6));
			bean.getClassBean().setMode(rs.getString(7));
			String sup = rs.getString(8);
			if (sup.equals("true")){
				bean.getClassBean().setSuperClass(true);
			} else {
				bean.getClassBean().setSuperClass(false);
			}
			bean.getClassBean().setManager(rs.getString(9));
			bean.getClassBean().setStatus(rs.getString(10));
			// attribute info
			bean.setName(rs.getString(11));
			bean.setSQLType(rs.getString(12));
			bean.setType(rs.getString(12)); // transform SQLType in aType, same getString
			bean.setLength(rs.getInt(13)); // attributelength (String length)
			bean.setMandatory(rs.getString(14));
			bean.setDefaultValue(rs.getString(15)); // default
			bean.setMode(rs.getString(16)); // mode
			bean.setDescription(rs.getString(17));
			bean.setLookup(rs.getString(18));
			bean.setReference(rs.getString(19));
			bean.setReferenceDomain(rs.getString(20));
			bean.setReferenceType(rs.getString(21));
			bean.setReferenceDirect(rs.getBoolean(22));
			bean.setDateExpire(rs.getBoolean(23)); 
			bean.setBaseDSP(rs.getString(24)); // baseDSP
			bean.setStatus(rs.getString(25));  // status
			bean.setIndex(rs.getInt(26)); // logical-index
			bean.setDBIndex(rs.getInt(27)); // db-index
			bean.setComment(rs.getString(28)); // attribute comment
			//DEBUG.println("ab name="+bean.getName()+" type8?="+bean.getType()+" ref="+bean.getReference());
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return bean;
	}
	
} // end of class
