
package cmdbuild.servlets;


import javax.servlet.*;
import javax.servlet.http.*;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperReport;

import java.sql.SQLException;
import java.util.*;
import java.io.*;

import cmdbuild.collection.AttributeBean;
import cmdbuild.collection.ErrorBean;
import cmdbuild.collection.ReportBean;
import cmdbuild.db.DBService;
import cmdbuild.elements.CMDBlookup;
import cmdbuild.elements.CMDBreport;
import cmdbuild.utils.Debug;
import cmdbuild.wizards.reportutils.ReportFactory;
import cmdbuild.wizards.reportutils.ReportInterface;
import cmdbuild.wizards.reportutils.WizardReportSerializable;

public class ReportFactoryController extends HttpServlet implements ReportInterface{
	
	static final long serialVersionUID = 1L;
	private int reportType;
	private int id;
	
	public void init(ServletConfig config) throws ServletException {
		super.init(config);
		
		System.setProperty("org.xml.sax.driver", "org.apache.xerces.parsers.SAXParser");
		System.setProperty("jasper.reports.compile.keep.java.file","false");
		System.setProperty("java.awt.headless","true");
		System.setProperty("sun.java2d.fontpath","$JAVA_HOME/jre/lib/fonts");
		String path = this.getServletContext().getRealPath("/WEB-INF/lib/jasperreports-1.2.2.jar")+System.getProperty("path.separator")+this.getServletContext().getRealPath("/WEB-INF/lib/")+System.getProperty("path.separator")+this.getServletContext().getRealPath("/report/reportTemp/");
		System.setProperty("jasper.reports.compile.class.path",path);
		System.setProperty("jasper.reports.compile.temp",this.getServletContext().getRealPath("/report/reportTemp/"));
	}
	
	public void destroy() {
	}
	
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
		//System.out.println("servlet tempDir="+this.getServletContext().getRealPath("/report/reportTemp/"));
		System.out.println("**REPORTFACTORYCONTROLLER**");
		Debug DEBUG = new Debug();
		String cmdbRealPath = (String) request.getSession().getAttribute("cmdbRealPath");
		DBService db = (DBService)this.getServletContext().getAttribute("db");
		CMDBreport REPORT = new CMDBreport(db);
		ReportBean rb = null;
		try {
			rb = REPORT.selectReportById(id);
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		ReportFactory rf = null;
		Vector atV = new Vector();
		if (rb.getType().equals(ReportBean.DEMO)){
			System.out.println("DEMO RFC type="+rb.getType());
			AttributeBean ab = null;
			// classid
			ab = new AttributeBean();
			ab.setName("classid");
			ab.setDescription("class id");
			ab.setType("int4");
			ab.setComment("comment");
			ab.getClassBean().setName("");
			ab.getClassBean().setComment("alias1");
			atV.add(ab);
			// classname
			ab = new AttributeBean();
			ab.setName("classname");
			ab.setDescription("class name");
			ab.setType("varchar");
			ab.setComment("comment");
			ab.getClassBean().setName("");
			ab.getClassBean().setComment("alias1");
			atV.add(ab);
			// classdescription
			ab = new AttributeBean();
			ab.setName("classdescription");
			ab.setDescription("class description");
			ab.setType("varchar");
			ab.setComment("comment");
			ab.getClassBean().setName("");
			ab.getClassBean().setComment("alias2");
			atV.add(ab);
			// classmode
			ab = new AttributeBean();
			ab.setName("classmode");
			ab.setDescription("class mode");
			ab.setType("varchar");
			ab.setComment("comment");
			ab.getClassBean().setName("");
			ab.getClassBean().setComment("alias2");
			atV.add(ab);
			rf = new ReportFactory(rb,cmdbRealPath,db,atV);
			rf.setReportType(reportType);
			rf.createReport();
			try {
				rf.compile();
			} catch (JRException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} else {
			//System.out.println("RFC type="+rb.getType());
			rf = new ReportFactory(cmdbRealPath,db);
			rf.setReportBean(rb);
			rf.setReportType(reportType);
			JasperReport rep = null;
			if (reportType==ReportFactory.CSV){ // simple report
				try {
					rep = (JasperReport) REPORT.getSimpleReport(id);
				} catch (SQLException e) {
					e.printStackTrace();
				}
			} else { // rich report pdf etc....
				try {
					rep = (JasperReport)REPORT.getRichReport(id);
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
			rf.setReport(rep);
		}
		try {
			rb.setWizard(REPORT.getWizard(id));
		} catch (SQLException e1) {
			e1.printStackTrace();
		}
		String fileOut = rf.getReportName();
		response.setContentType(rf.getContentType());
		ServletOutputStream out = null;
		out = response.getOutputStream();
		try {
			if (rb.getType().equals(ReportBean.DEMO)){
				rf.fill();	
			} else {
				AttributeBean ab = null;
				String str = "";
				WizardReportSerializable wrs = (WizardReportSerializable) rb.getWizard();
				Vector orders = wrs.getQueryFactory().getOrders();
				// find if there are lookups
				Hashtable map = new Hashtable();
				Vector lookupTypes = new Vector();
				HashSet hash = new HashSet();
				Hashtable referenceHash = new Hashtable(); 
				CMDBlookup LOOKUP = new CMDBlookup(db);
				Vector options = null;
				for (int i=0;i<orders.size();i++){
					ab = (AttributeBean) orders.elementAt(i);
					str = (ab.getClassBean().getComment()+ab.getName()).toLowerCase();
					//System.out.println("map add "+str+" type="+ab.getType()+" lookup="+ab.getLookup());
					map.put(str,ab);
					if (ab.getType()==AttributeBean.LOOKUP){
						hash.add(ab.getLookup());
					}
					if (ab.getType()==AttributeBean.REFERENCE){
						try {
							if (!referenceHash.containsKey(ab.getReference()) && ab.getReference().length()>0){ // don't get duplicates
								DEBUG.println("Getting refrence for "+ab.getReference());
								options = LOOKUP.selectAllReference(ab.getReference());
								referenceHash.put(ab.getReference(),options);
							}
						} catch (SQLException e) {
							e.printStackTrace();
						}
					}
				}
				lookupTypes.addAll(hash);
				Vector sortedOrders = new Vector();
				String query = rb.getQuery();
				// get the right order of alias
				query = query.substring(0,query.indexOf("FROM"));
				StringTokenizer tok = new StringTokenizer(query,",",false);
				while (tok.hasMoreTokens()){
					str = tok.nextToken();
					str = str.substring(str.indexOf("AS")+2,str.length());
					str = str.trim();
					//System.out.println("str="+str+"="+map.get(str));
					ab = (AttributeBean) map.get(str);
					if (ab!=null){
						sortedOrders.add(ab);
					} 
					//sorted.add(str);
				}
				//System.out.println("sortedOrders="+sortedOrders.size()+" lookupTypes.size()="+lookupTypes.size());
				Vector lookupVector = new Vector();
				if (lookupTypes.size()>0){
					lookupVector = LOOKUP.selectLookupByTypes(lookupTypes);
				}	
				
				//System.out.println("replace lookups.....");
				if (lookupVector.size()>0 || referenceHash.size()>0){
					rf.fillWith(rb.getQuery(),sortedOrders,lookupVector,referenceHash);
					//rf.fill();
				} else {
					rf.fill();
				}
			}
			//rf.sendReportToFile(fileOut);
			rf.sendReportToStream(fileOut,out);
			out.flush();
			out.close();
		} catch (Exception e) {
			e.printStackTrace();
			//sendError(e,request,response);
		} 
	}
	
	public void sendError(Exception e, HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
		ErrorBean err = new ErrorBean();
		err.setException(e);
		err.setFromPage("reportpopup.jsp");
		request.getSession().setAttribute("error",err);			
		RequestDispatcher rd = request.getRequestDispatcher("error.jsp");
		rd.forward(request,response);		
	}

	public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
		String repType = "";
		if (request.getParameter("type")!=null){
			repType = request.getParameter("type");
		}
		String sid = request.getParameter("id");
		id = Integer.parseInt(sid);
		reportType = 0;
		if (repType.equals("csv")){
			reportType = CSV;
		} else {
			reportType = PDF;
		}
		doPost(request,response);
	}
	
} // end of class	

