package cmdbuild.wizards.reportutils;

import cmdbuild.db.DBService;
import cmdbuild.elements.CMDBreport;

import java.io.File;
import java.io.OutputStream;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import cmdbuild.collection.AttributeBean;
import cmdbuild.collection.ReportBean;

import net.sf.jasperreports.engine.JRBand;
import net.sf.jasperreports.engine.JRElement;
import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JRExporter;
import net.sf.jasperreports.engine.JRExporterParameter;
import net.sf.jasperreports.engine.JRExpression;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperExportManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.data.JRTableModelDataSource;
import net.sf.jasperreports.engine.design.JRDesignExpression;
import net.sf.jasperreports.engine.design.JRDesignField;
import net.sf.jasperreports.engine.design.JRDesignImage;
import net.sf.jasperreports.engine.design.JRDesignQuery;
import net.sf.jasperreports.engine.design.JRDesignStaticText;
import net.sf.jasperreports.engine.design.JRDesignTextField;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.export.JRCsvExporter;
import net.sf.jasperreports.engine.export.JRCsvExporterParameter;
import net.sf.jasperreports.engine.export.JRPdfExporter;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

public class ReportFactory implements ReportInterface{
	
	private Vector attributeVector; // fields (list of AttributeBean)
	private String querySQL;
	private DBService db;
	private ReportBean reportBean;
	// Directory
	private String outDir;
	private String tempDir;
	// Jasper
	private JasperDesign design;
	private JasperReport report;
	private JasperPrint print;
	private int reportType;
	// data to update
	private Vector columnVector;
	private Vector detailVector;
	private int pageWidth;
	
	// to create a report from template
	public ReportFactory(ReportBean rb , String cmdbRealpath, DBService db, Vector attributeVector) {
		super();
		this.outDir = cmdbRealpath+File.separator+"report";
		this.tempDir = outDir+File.separator+"reportTemp";
		this.attributeVector = attributeVector;
		this.querySQL = rb.getQuery();
		this.reportBean = rb;
		this.db = db;
		this.pageWidth = 0;
	}
	// to use a report already created
	public ReportFactory(String cmdbRealpath, DBService db) {
		super();
		this.outDir = cmdbRealpath+File.separator+"report";
		this.tempDir = outDir+File.separator+"reportTemp";
		System.out.println("Report Factory outDir="+outDir+" tempdir="+tempDir);
		this.db = db;
	}
	
	/*
	 * Report functions
	 */
	public void compile() throws JRException{
		long start = System.currentTimeMillis();
			report = JasperCompileManager.compileReport(design);
		System.out.println("REPORT compilation time="+( System.currentTimeMillis()-start) );
	}
	public void fill() throws JRException{
		if (report==null){
			System.out.println("filling a null report!");
		}
		Map parameters = new HashMap();
		parameters.put("REPORT_CONNECTION",db.getConnection());
		long start = System.currentTimeMillis();
		print = JasperFillManager.fillReport(report, parameters, db.getConnection());
		System.out.println("REPORT fill time="+( System.currentTimeMillis()-start) );
	}
	public void fillWith(String query, Vector attributeVector, Vector lookupVector, Hashtable referenceHash) throws JRException, SQLException{
		if (report==null){
			System.out.println("filling a null report!");
		}
		Map parameters = new HashMap();
		parameters.put("REPORT_CONNECTION",db.getConnection());
		long start = System.currentTimeMillis();
		CMDBreport REPORT = new CMDBreport(db); 
		Vector rows = REPORT.executeReportQuery(query,attributeVector,lookupVector,referenceHash);
		print = JasperFillManager.fillReport(report, parameters,  new JRTableModelDataSource(new ReportTableModel(attributeVector,rows)));
		System.out.println("REPORT fill time="+( System.currentTimeMillis()-start) );
	}
	public void sendReportToFile(String fileOut) throws Exception {
		switch( reportType ) {
			case PDF:	JasperExportManager.exportReportToPdfFile(print,outDir+File.separator+fileOut+".pdf");
						break;
			case CSV:	JRCsvExporter csvExporter = new JRCsvExporter();
						csvExporter.setParameter(JRExporterParameter.JASPER_PRINT, print);
						csvExporter.setParameter(JRExporterParameter.OUTPUT_FILE_NAME,outDir+File.separator+fileOut+".csv");
						csvExporter.setParameter(JRCsvExporterParameter.FIELD_DELIMITER, ";"); 
						csvExporter.exportReport();
						break;
		}
	}
	public void sendReportToStream(String fileOut, OutputStream outStream) throws Exception, Error {
		JRExporter exporter = null;
		switch( reportType ) {
		case PDF:	exporter = new  JRPdfExporter();
					exporter.setParameter(JRExporterParameter.JASPER_PRINT, print);
					exporter.setParameter(JRExporterParameter.OUTPUT_STREAM, outStream);	
					exporter.setParameter(JRExporterParameter.OUTPUT_FILE_NAME,fileOut+".pdf");
					//exporter.setParameter(JRHtmlExporterParameter.IMAGES_URI, tempDir); 
					break;
		case CSV:	exporter = new JRCsvExporter();
					exporter.setParameter(JRExporterParameter.JASPER_PRINT, print);
					exporter.setParameter(JRExporterParameter.OUTPUT_STREAM, outStream);	
					exporter.setParameter(JRExporterParameter.OUTPUT_FILE_NAME,fileOut+".csv");
					exporter.setParameter(JRCsvExporterParameter.FIELD_DELIMITER, ";"); 
					break;
		}
		exporter.exportReport();
	}
	
	/*	
	 * design functions
	 */
	public void createReport(){
		try {
			if (reportType==CSV){
				design = JRXmlLoader.load(tempDir+File.separator+CSV_TEMPLATE);
				//System.out.println("Template="+CSV_TEMPLATE);
			} else {
				design = JRXmlLoader.load(tempDir+File.separator+PDF_TEMPLATE);
				pageWidth = design.getPageWidth();
				//System.out.println("Template="+PDF_TEMPLATE);
			}
			JRDesignQuery query = new JRDesignQuery();
			query.setText(querySQL);
			design.setQuery(query);
			if (reportType!=CSV){ // PDF
				modifyTitle();
				modifyFields();
				modifyColumnHeader();
				modifyDetail();
				updateDesign();
			} else { // CSV
				modifyFields();
				modifyDetail();
				//design.
			}
		} catch (JRException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
	}	
	private void modifyTitle() throws JRException{
		JRBand title = design.getTitle();
		List f = title.getChildren();
		Iterator it = f.iterator();
		JRDesignImage img = null;
		JRExpression varExpr = null;
		JRDesignExpression varExp = null;
		JRDesignStaticText field = null;
		Object obj = null;
		String path = "";
		char sep = '\\';
		StringBuffer newpath = null;
		// Setting path and escaping (if windows)
		while (it.hasNext()){
			obj = it.next();
			if (obj instanceof JRDesignImage) {
				newpath = new StringBuffer();
				img = (JRDesignImage) obj;
				varExpr = img.getExpression();
				varExp = (JRDesignExpression) varExpr;
				path = "\""+tempDir+File.separator+varExp.getText().substring(1,varExp.getText().length()-1)+"\"";
				//System.out.println("image path="+path);
				// windows separator needs to be escaped
				if (File.separator.toCharArray()[0]==sep){
					char[] ca = path.toCharArray();
					char ct;
					for (int i=0;i<ca.length;i++){
						ct = ca[i];
						if (ct!=sep){
							newpath.append(ct);
						} else {
							newpath.append(sep);
							newpath.append(ct);
						}
					}
					//System.out.println("windows path="+path);
					path = newpath.toString();
				}
				System.out.println("set image path="+path);
				varExp.setText(path);
			} else if (obj instanceof JRDesignStaticText){
				// setting title
				field = (JRDesignStaticText) obj;
				field.setText(reportBean.getName());
			}
		}
	}
	private void modifyFields() throws JRException{
		List f = design.getFieldsList();
		Vector vett = new Vector(f);
		JRDesignField field = null;
		Enumeration en = vett.elements();
		// removing default filds
		while (en.hasMoreElements()){
			field = (JRDesignField) en.nextElement();
			design.removeField(field);
		}
		AttributeBean ab = null;
		en = attributeVector.elements();
		while (en.hasMoreElements()){
			ab = (AttributeBean) en.nextElement();
			field = setField(ab);
			design.addField(field);
		}
	}
	private void modifyColumnHeader(){
		JRBand columnHeader = design.getColumnHeader();
		JRElement[] elements = columnHeader.getElements();
		columnVector = new Vector();
		Vector graphicVector = new Vector();
		// separate elements 
		for (int i=0;i<elements.length;i++ ) {
			if (elements[i] instanceof JRDesignStaticText) {
				columnVector.add(elements[i]);
			} else { // save graphic elements
				graphicVector.add(elements[i]);
			}
		}
		// align dimensions of vectors
		JRDesignStaticText field = null;
		int af = attributeVector.size();
		int df = columnVector.size();
		AttributeBean ab = null;
		if (af!=df){
			if (af>=df){
				for (int i=0; i<af; i++){
					if (i<df){ // modify the field
						ab = (AttributeBean) attributeVector.elementAt(i);
						field = (JRDesignStaticText) columnVector.elementAt(i);
					} else { // add a filed
						ab = (AttributeBean) attributeVector.elementAt(i);
						field = new JRDesignStaticText();
						columnVector.add(field);
					}
				}
			} else { // af<df
				// delete extra fields
				for (int i=df-1;i>af-1;i--){
					columnVector.removeElementAt(i);
				}
				df = columnVector.size();
				for (int i=0; i<df; i++){
					if (i<af){ // modify the field
						ab = (AttributeBean) attributeVector.elementAt(i);
						field = (JRDesignStaticText) columnVector.elementAt(i);
					} else { // delete a field
						System.out.println("ReportFactory  alignVectors error");
					}
				}
			}
		}
		Enumeration en = columnVector.elements();
		JRDesignStaticText dst = null;
		int i = 0;
		ab = null;
		// setting titles
		while (en.hasMoreElements()){
			dst = (JRDesignStaticText) en.nextElement();
			ab = (AttributeBean) attributeVector.elementAt(i);
			if (ab.getDefaultValue().length()>0){
				dst.setText(ab.getDefaultValue());
			} else {
				dst.setText(ab.getClassBean().getDescription()+" "+ab.getDescription());	
			}
			if (ab.getBaseDSP().length()>0){
				if (ab.getBaseDSP().equals("center")){
					dst.setHorizontalAlignment(JRDesignTextField.HORIZONTAL_ALIGN_CENTER);	
				} else if (ab.getBaseDSP().equals("right")){
					dst.setHorizontalAlignment(JRDesignTextField.HORIZONTAL_ALIGN_RIGHT);
				} else if (ab.getBaseDSP().equals("justify")){
					dst.setHorizontalAlignment(JRDesignTextField.HORIZONTAL_ALIGN_JUSTIFIED);
				} else { // left, default
					// dst.setHorizontalAlignment(JRDesignTextField.HORIZONTAL_ALIGN_CENTER);
				}
			}
			i++;
		}
		// saving new list of items
		List save = columnHeader.getChildren();
		save.clear();
		en = graphicVector.elements();
		while (en.hasMoreElements()){
			save.add(en.nextElement());
		}
		en = columnVector.elements();
		while (en.hasMoreElements()){
			save.add(en.nextElement());
		}
	}
	private void modifyDetail(){
		JRBand detail = design.getDetail();
		List f = detail.getChildren();
		Iterator it = f.iterator();
		detailVector = new Vector();
		Vector graphicVector = new Vector();
		Object obj = null;
		while (it.hasNext()){
			obj = it.next();
			if (obj instanceof JRDesignTextField) {
				detailVector.add(obj);
			} else {
				graphicVector.add(obj);
			}
		}
		JRDesignTextField field = null;
		int af = attributeVector.size();
		int df = detailVector.size();
		AttributeBean ab = null;
		if (af!=df){
			if (af>=df){
				for (int i=0; i<af; i++){
					if (i<df){ // modify the field
						ab = (AttributeBean) attributeVector.elementAt(i);
						field = (JRDesignTextField) detailVector.elementAt(i);
						setTextField(ab,field);
					} else { // add a filed
						ab = (AttributeBean) attributeVector.elementAt(i);
						field = new JRDesignTextField();
						setTextField(ab,field);
						detailVector.add(field);
					}
				}
			} else { // af<df
				// delete extra fields
				for (int i=df-1;i>af-1;i--){
					detailVector.removeElementAt(i);
				}
				df = detailVector.size();
				for (int i=0; i<df; i++){
					if (i<af){ // modify the field
						ab = (AttributeBean) attributeVector.elementAt(i);
						field = (JRDesignTextField) detailVector.elementAt(i);
						setTextField(ab,field);
					} else { // delete a field
						System.out.println("ReportFactory  alignVectors error");
					}
				}
			}
		}
		Enumeration en = detailVector.elements();
		int i = 0;
		ab = null;
		// setting titles
		while (en.hasMoreElements()){
			field = (JRDesignTextField) en.nextElement();
			ab = (AttributeBean) attributeVector.elementAt(i);
			setTextField(ab,field);
			i++;
		}
		// saving new list of items
		List save = detail.getChildren();
		save.clear();
		en = graphicVector.elements();
		while (en.hasMoreElements()){
			save.add(en.nextElement());
		}
		en = detailVector.elements();
		while (en.hasMoreElements()){
			save.add(en.nextElement());
		}
	}
	// update position of elements
	private void updateDesign(){
		// calculate weight of all elements
		Hashtable weight = new Hashtable();
		AttributeBean ab = null;
		int virtualWidth = 0;
		int size = 0;
		String key = "";
		for (int i=0;i<attributeVector.size();i++){
			ab = (AttributeBean) attributeVector.elementAt(i);
			switch (ab.getType()) {
   			case AttributeBean.BOOLEAN:    size=4;break;
   		 	case AttributeBean.INTEGER:    size=8;break;
     		case AttributeBean.DECIMAL:    size=8;break;
   			case AttributeBean.DOUBLE:     size=8;break;
     		case AttributeBean.DATE:       size=10;break;
		   	case AttributeBean.TIMESTAMP:  size=16;break;
		   	case AttributeBean.LOOKUP:     size=20;break;
	      	case AttributeBean.REFERENCE:  size=20;break;
	       	case AttributeBean.TEXT:       size=50;break;
   		  	case AttributeBean.STRING:     size=(ab.getLength()>4?ab.getLength():4)>40?40:(ab.getLength()>4?ab.getLength():4);break;
	       	default	:			           size=1;break;
			}    	
			virtualWidth += size;
			weight.put(ab.getClassBean().getComment()+ab.getName(),size+"");
			if (ab.getDefaultValue().length()>0){
				key = ab.getDefaultValue();
			} else {
				key = ab.getClassBean().getDescription()+" "+ab.getDescription();	
			}
			weight.put(key,size+"");
		}	
		double cx = ((double)pageWidth*0.95) / ((double)virtualWidth);
		System.out.println("cx="+cx+" pageWidth "+((double)pageWidth*0.95) +" / virtualWidth "+virtualWidth);
		double doub = 0;
		JRBand detail = design.getDetail();
		JRElement[] elements = detail.getElements();
		JRDesignTextField dtf = null;
		int x = 0;
		System.out.println("RF updateDesign DESIGN");
		JRDesignExpression varExpr = null;
		for (int i=0;i<elements.length;i++ ) {
			if (elements[i] instanceof JRDesignTextField) {
				dtf = (JRDesignTextField) elements[i];
				varExpr = (JRDesignExpression) dtf.getExpression();
				key = varExpr.getText();
				System.out.println("text="+key);
				key = key.substring(3,key.length()-1);
				System.out.println("text="+key);
				key = (String) weight.get(key);
				System.out.println("kry="+key);
				try {
					size = Integer.parseInt(key);
				} catch (NumberFormatException e) {
					size = 0;
				}
				doub = ((double)size)*cx ;
				size = (int) doub;
				dtf.setX(x);
				dtf.setWidth(size);
				System.out.println("RF updateDesign x="+dtf.getX()+" Width="+dtf.getWidth());
				x += size;
			} 
		}
		// sizing table headers
		JRBand columnHeader = design.getColumnHeader();
		elements = columnHeader.getElements();
		JRDesignStaticText dst = null;
		x = 0;
		System.out.println("RF updateDesign HEADER");
		for (int i=0;i<elements.length;i++ ) {
			if (elements[i] instanceof JRDesignStaticText) {
				dst = (JRDesignStaticText) elements[i];
				key = dst.getText();
				System.out.println("text="+key);
				key = (String) weight.get(key);
				System.out.println("kry="+key);
				try {
					size = Integer.parseInt(key);
				} catch (NumberFormatException e) {
					size = 0;
				}
				doub = ((double)size)*cx ;
				size = (int) doub;
				dst.setX(x);
				dst.setWidth(size);
				System.out.println("RF updateDesign"+dst.getText()+" x="+dst.getX()+" Width="+dst.getWidth());
				x += size;
			}
		}
	}
	/*
	// update position of elements
	private void updateDesign(){
		int nelem = attributeVector.size();
		int size = pageWidth / nelem;
		System.out.println("RF updateDesign pageWidth="+pageWidth+"/ nelem="+nelem+" ==size="+size);
		// sizing table headers
		JRBand columnHeader = design.getColumnHeader();
		JRElement[] elements = columnHeader.getElements();
		JRDesignStaticText dst = null;
		int j = 0;
		System.out.println("RF updateDesign HEADER");
		for (int i=0;i<elements.length;i++ ) {
			if (elements[i] instanceof JRDesignStaticText) {
				dst = (JRDesignStaticText) elements[i];
				dst.setX(j*size);
				dst.setWidth(size);
				System.out.println("RF updateDesign"+dst.getText()+" x="+dst.getX()+" Width="+dst.getWidth());
				j++;
			}
		}
		JRBand detail = design.getDetail();
		elements = detail.getElements();
		JRDesignTextField dtf = null;
		j = 0;
		System.out.println("RF updateDesign DESIGN");
		for (int i=0;i<elements.length;i++ ) {
			if (elements[i] instanceof JRDesignTextField) {
				dtf = (JRDesignTextField) elements[i];
				dtf.setX(j*size);
				dtf.setWidth(size);
				System.out.println("RF updateDesign x="+dtf.getX()+" Width="+dtf.getWidth());
				j++;
			}
		}
	}
	*/
	/*
	 * Setters of fields functions
	 */
	private  JRDesignField setField(AttributeBean ab){
		JRDesignField field = new JRDesignField();
		if (ab.getType()==AttributeBean.DOUBLE || ab.getType()==AttributeBean.DECIMAL){
			field.setName(ab.getClassBean().getComment()+ab.getName());
			field.setDescription(ab.getClassBean().getComment()+ab.getName());
			field.setValueClassName("java.lang.Double");
		} else if (ab.getType()==AttributeBean.INTEGER){
			field.setName(ab.getClassBean().getComment()+ab.getName());
			field.setDescription(ab.getClassBean().getComment()+ab.getName());
			field.setValueClassName("java.lang.Integer");
		} else {
			field.setName(ab.getClassBean().getComment()+ab.getName());
			field.setDescription(ab.getClassBean().getComment()+ab.getName());
			field.setValueClassName("java.lang.String");
		}
		return field;
	}
	private void setTextField(AttributeBean ab, JRDesignTextField field){
		JRDesignExpression varExpr = null;
		if (ab.getType()==AttributeBean.DOUBLE || ab.getType()==AttributeBean.DECIMAL){
			varExpr = new JRDesignExpression();
			varExpr.setValueClassName("java.lang.Double");
			varExpr.setText("$F{"+ab.getClassBean().getComment()+ab.getName()+"}");
			field.setExpression(varExpr);
		} else if (ab.getType()==AttributeBean.INTEGER){
			varExpr = new JRDesignExpression();
			varExpr.setValueClassName("java.lang.Integer");
			varExpr.setText("$F{"+ab.getClassBean().getComment()+ab.getName()+"}");
			field.setExpression(varExpr);
		} else {
			varExpr = new JRDesignExpression();
			varExpr.setValueClassName("java.lang.String");
			varExpr.setText("$F{"+ab.getClassBean().getComment()+ab.getName()+"}");
			field.setExpression(varExpr);
		}
		if (ab.getBaseDSP().length()>0){
			if (ab.getBaseDSP().equals("center")){
				field.setHorizontalAlignment(JRDesignTextField.HORIZONTAL_ALIGN_CENTER);	
			} else if (ab.getBaseDSP().equals("right")){
				field.setHorizontalAlignment(JRDesignTextField.HORIZONTAL_ALIGN_RIGHT);
			} else if (ab.getBaseDSP().equals("justify")){
				field.setHorizontalAlignment(JRDesignTextField.HORIZONTAL_ALIGN_JUSTIFIED);
			} else { // left, default
				// dst.setHorizontalAlignment(JRDesignTextField.HORIZONTAL_ALIGN_CENTER);
			}
		}
		if (ab.getMode().length()>0){
			field.setPattern(ab.getMode());
		}
	}
  	public String getContentType(){
		if (reportType==PDF) {
			return "application/pdf";
		} else if (reportType==CSV) {
			return "text/plain";
		} else {
			return "";
		}
  	}
  	public int getReportType() {
		return reportType;
	}
	public void setReportType(int reportType) {
		this.reportType = reportType;
	}
	public String getReportName() {
		return reportBean.getName();
	}
	public void setReport(JasperReport report) {
		if (report==null){
			System.out.println("setReport report is null!");
		}
		this.report = report;
	}
	public JasperReport getReport() {
		return report;
	}
	public ReportBean getReportBean() {
		return reportBean;
	}
	public void setReportBean(ReportBean reportBean) {
		this.reportBean = reportBean;
	}
	public String getQuerySQL() {
		return querySQL;
	}
	
  	
} // end of class
