package cmdbuild.wizards.reportutils;

import cmdbuild.collection.AttributeBean;
import cmdbuild.collection.ClassBean;
import cmdbuild.collection.LookupBean;
import cmdbuild.utils.MultiLanguage;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

public class ReportFilter implements Serializable{

	static final long serialVersionUID = 1L;
	private Vector classVector;
	private Vector attributeVector;
	private Vector lookupVector;
	private Hashtable referenceHash;
	private int row;
	private int subPage;
	private String[] valuesFromRequest;
	// valuesFromRequest mapping
	public final static int TARGET_CLASS 		= 0;
	public final static int TARGET_ATTRIBUTE	= 1;
	public final static int FIRST_OPERATION 	= 2;
	public final static int FIRST_COMPARISON 	= 3;
	public final static int AND_OR				= 4;
	public final static int LAST_OPERATION	 	= 5;
	public final static int LAST_COMPARISON		= 6;
	private String alias;  // query alias
	private boolean halfValid;
	private boolean fullValid;
	private AttributeBean attributeBean;
 	private MultiLanguage multilanguage;
 	
 	public ReportFilter(MultiLanguage multilanguage, Vector classVector, int row) {
		super();
		// TODO Auto-generated constructor stub
		this.classVector = classVector;
		this.row = row;
		this.valuesFromRequest = new String[7];
		this.halfValid = false;
		this.fullValid = false;
		this.multilanguage = multilanguage;
		this.subPage = 0;
		clearValues();
	}

	// Vectors of String parameters for the request
	public Vector getRequestParam(){
		Vector res = new Vector();
		String paramname= "";
		for (int i=0;i<=6;i++){
			paramname = "r"+row+"d"+i;
			res.add(paramname);
		}
		return res;
	}
	/*
	 * Values
	 */
	public void clearValues(){
		for (int i=0;i<valuesFromRequest.length;i++){
			valuesFromRequest[i]="";
		}
	}
	public void setValueAt(String val, int pos){
		valuesFromRequest[pos]=val;
	}
	public void showValues(){
		String debug = "";
		for (int i=0;i<valuesFromRequest.length;i++){
			debug += "%"+valuesFromRequest[i];
		}
		System.out.println("rf DEBUG vales="+debug);
	}
	/*
	 * Request
	 */
	public String getRequestAttribute(){
		return (String) valuesFromRequest[TARGET_ATTRIBUTE];
	}
	public String getRequestClass(){
		String result = "";
		int c = 0;
		if (valuesFromRequest[TARGET_CLASS].length()>0 && valuesFromRequest[TARGET_CLASS].startsWith("c")){
			result = (String) valuesFromRequest[TARGET_CLASS];
			result = result.substring(5,result.length());
			c = Integer.parseInt(result);
			c--;
			result = ( (ClassBean)classVector.elementAt(c)).getName();
		}
		return result;
	}
	
	// map class1, class2 etc.. into query alias computer1,location2 etc....
	public void mapAlias(Vector aliases){
		Hashtable h = new Hashtable();
		Enumeration en = aliases.elements();
		ClassBean cb = null;
		int i = 1;
		while (en.hasMoreElements()){
			cb = (ClassBean) en.nextElement();
			h.put("class"+i,cb.getComment());
			//System.out.println("MAPaliases "+cb.getName()+" class"+i+"="+cb.getComment());
			i++;
		}
		alias = (String) h.get(valuesFromRequest[TARGET_CLASS]);
		//System.out.println("target class="+valuesFromRequest[TARGET_CLASS]+" alias getted is="+alias);
		//System.out.println("mapAlias mapping "+valuesFromRequest[TARGET_CLASS]+"into alias="+alias);
	}
	public void validate(){
		halfValid = false;
		fullValid = false;
		if (valuesFromRequest[TARGET_CLASS].length()>0 && valuesFromRequest[TARGET_CLASS].startsWith("c")){
			if (valuesFromRequest[TARGET_ATTRIBUTE].length()>0 && valuesFromRequest[FIRST_OPERATION].length()>0 && valuesFromRequest[FIRST_COMPARISON].length()>0){
				halfValid = true;
			}
			if (halfValid && valuesFromRequest[LAST_OPERATION].length()>0 && valuesFromRequest[LAST_COMPARISON].length()>0){
				fullValid = true;
			}
		}
	}
	/*
	 * toString functions
	 */
	public String toQuery(){
		String query = "";
		String up = "";
		if (halfValid){
			//System.out.println("rf toQuery attributeBean="+attributeBean.getName()+" type="+attributeBean.getType());
			if (attributeBean.getType()==AttributeBean.STRING || attributeBean.getType()==AttributeBean.DATE || attributeBean.getType()==AttributeBean.TIMESTAMP){
				if (valuesFromRequest[FIRST_OPERATION].equals("eq")){
					query = "UPPER("+alias+".\""+valuesFromRequest[TARGET_ATTRIBUTE]+"\")='"+valuesFromRequest[FIRST_COMPARISON]+"'";
				} else if (valuesFromRequest[FIRST_OPERATION].equals("con")){
					up = valuesFromRequest[FIRST_COMPARISON].toString().toUpperCase();
					query = "UPPER("+alias+".\""+valuesFromRequest[TARGET_ATTRIBUTE]+"\") LIKE '%"+up+"%'";
				} else if (valuesFromRequest[FIRST_OPERATION].equals("sta")){
					up = valuesFromRequest[FIRST_COMPARISON].toString().toUpperCase();
					query = "UPPER("+alias+".\""+valuesFromRequest[TARGET_ATTRIBUTE]+"\") LIKE '"+up+"%'";
				}
			} else{
				if (valuesFromRequest[FIRST_OPERATION].equals("eq")){
					query = alias+".\""+valuesFromRequest[TARGET_ATTRIBUTE]+"\"="+valuesFromRequest[FIRST_COMPARISON]+"";
				} else if (valuesFromRequest[FIRST_OPERATION].equals("min")){
					query = alias+".\""+valuesFromRequest[TARGET_ATTRIBUTE]+"\"<"+valuesFromRequest[FIRST_COMPARISON]+"";
				} else if (valuesFromRequest[FIRST_OPERATION].equals("mag")){
					query = alias+".\""+valuesFromRequest[TARGET_ATTRIBUTE]+"\">"+valuesFromRequest[FIRST_COMPARISON]+"";
				}
			}
		}
		if (fullValid){
			//System.out.println("rf toQuery attributeBean="+attributeBean.getName()+" type="+attributeBean.getType());
			if (attributeBean.getType()==AttributeBean.STRING || attributeBean.getType()==AttributeBean.DATE || attributeBean.getType()==AttributeBean.TIMESTAMP){
				if (valuesFromRequest[LAST_OPERATION].equals("eq")){
					query += " "+valuesFromRequest[AND_OR]+" "+"UPPER("+alias+".\""+valuesFromRequest[TARGET_ATTRIBUTE]+"\")='"+valuesFromRequest[LAST_COMPARISON]+"'";
				} else if (valuesFromRequest[LAST_OPERATION].equals("con")){
					up = valuesFromRequest[LAST_COMPARISON].toString().toUpperCase();
					query += " "+valuesFromRequest[AND_OR]+" "+"UPPER("+alias+"."+valuesFromRequest[TARGET_ATTRIBUTE]+"\") LIKE '%"+up+"%'";
				} else if (valuesFromRequest[LAST_OPERATION].equals("sta")){
					up = valuesFromRequest[LAST_COMPARISON].toString().toUpperCase();
					query += " "+valuesFromRequest[AND_OR]+" "+"UPPER("+alias+"."+valuesFromRequest[TARGET_ATTRIBUTE]+"\") LIKE '"+up+"%'";
				}
			} else{
				if (valuesFromRequest[LAST_OPERATION].equals("eq")){
					query += " "+valuesFromRequest[AND_OR]+" "+alias+".\""+valuesFromRequest[TARGET_ATTRIBUTE]+"\"="+valuesFromRequest[LAST_COMPARISON]+"";
				} else if (valuesFromRequest[LAST_OPERATION].equals("min")){
					query += " "+valuesFromRequest[AND_OR]+" "+alias+".\""+valuesFromRequest[TARGET_ATTRIBUTE]+"\"<"+valuesFromRequest[LAST_COMPARISON]+"";
				} else if (valuesFromRequest[LAST_OPERATION].equals("mag")){
					query += " "+valuesFromRequest[AND_OR]+" "+alias+".\""+valuesFromRequest[TARGET_ATTRIBUTE]+"\">"+valuesFromRequest[LAST_COMPARISON]+"";
				}
			}
		}
		if (query.length()>0){
			query = "("+query+")";
		}
		return query;
	}
	// return a <tr>....</tr>
	public String toForm(){
		String HTMLtext = "";
		validate();
		Enumeration en = null;
		HTMLtext += "\t<tr>";
		// TARGET_CLASS
		HTMLtext += "<td id=\"r"+row+"c0\">";
		HTMLtext += "<select size=\"1\" name=\"r"+row+"d0\" onChange=\"choose("+row+")\">";
		HTMLtext += "   <option value=\"\">"+multilanguage.getMessage("com.choose")+"...</option>";
		en = classVector.elements();
		ClassBean cb = null;
		AttributeBean ab = null;
		LookupBean lb = null;
		Vector vett = null;
		int i = 1;
		while (en.hasMoreElements()){
			cb = (ClassBean) en.nextElement();
			// array of data are named as class1, class2 etc...
			if (valuesFromRequest[TARGET_CLASS].equals("class"+i) && halfValid){
				HTMLtext += "   <option value=\"class"+i+"\" selected>"+cb.getDescription()+"</option>";
			} else {
				HTMLtext += "   <option value=\"class"+i+"\">"+cb.getDescription()+"</option>";
			}
			i++;
		}
		HTMLtext += "</select>";
		HTMLtext += "</td>";
		// TARGET_ATTRIBUTE
		if (valuesFromRequest[TARGET_ATTRIBUTE].length()==0){
			HTMLtext += "<td id=\"r"+row+"c1\">";
			HTMLtext += "<select size=\"1\" name=\"r"+row+"d1\" onChange=\"choosetype("+row+")\">";
			HTMLtext += "   <option value=\"\"></option>";
			HTMLtext += "</select>";
			HTMLtext += "</td>";
		} else {
			HTMLtext += "<td id=\"r"+row+"c1\">";
			HTMLtext += "<select size=\"1\" name=\"r"+row+"d1\" onChange=\"choosetype("+row+")\">";
			en = attributeVector.elements();
			while (en.hasMoreElements()){
				ab = (AttributeBean) en.nextElement();
				if (ab.getName().equals(valuesFromRequest[TARGET_ATTRIBUTE]) && halfValid ){
					HTMLtext += "   <option value=\""+ab.getName()+"\" selected>"+ab.getDescription()+"</option>";
				} else {
					HTMLtext += "   <option value=\""+ab.getName()+"\">"+ab.getDescription()+"</option>";
				}
			}
			HTMLtext += "</select>";
			HTMLtext += "</td>";
		}
		if (!halfValid){
			// FIRST_OPERATION
			HTMLtext += "<td id=\"r"+row+"c2\">";
			HTMLtext += "<select size=\"1\" name=\"r"+row+"d2\">";
			HTMLtext += "   <option value=\"\"></option>";
			HTMLtext += "</select>";
			HTMLtext += "</td>";
			// FIRST_COMPARISON
			HTMLtext += "<td id=\"r"+row+"c3\">";
			HTMLtext += "<select size=\"1\" name=\"r"+row+"d3\">";
			HTMLtext += "   <option value=\"\"></option>";
			HTMLtext += "</select>";
			HTMLtext += "</td>";
		} else {
			// FIRST_OPERATION
			HTMLtext += "<td id=\"r"+row+"c2\">";
			HTMLtext += "<select size=\"1\" name=\"r"+row+"d2\">";
			if (attributeBean.getType()==AttributeBean.STRING){
				if (valuesFromRequest[FIRST_OPERATION].equals("eq")){
					HTMLtext += "   <option value=\"eq\" selected>"+multilanguage.getMessage("com.equal")+"</option>";
				} else {
					HTMLtext += "   <option value=\"eq\">"+multilanguage.getMessage("com.equal")+"</option>";
				}
				if (valuesFromRequest[FIRST_OPERATION].equals("con")){
					HTMLtext += "   <option value=\"con\" selected>"+multilanguage.getMessage("com.contains")+"</option>";
				} else {
					HTMLtext += "   <option value=\"con\">"+multilanguage.getMessage("com.contains")+"</option>";
				}
				if (valuesFromRequest[FIRST_OPERATION].equals("sta")){
					HTMLtext += "   <option value=\"sta\" selected>"+multilanguage.getMessage("com.startswith")+"</option>";
				} else {
					HTMLtext += "   <option value=\"sta\">"+multilanguage.getMessage("com.startswith")+"</option>";
				}
			} else {
				if (valuesFromRequest[FIRST_OPERATION].equals("eq")){
					HTMLtext += "   <option value=\"eq\" selected>"+multilanguage.getMessage("com.equal")+"</option>";	
				} else {
					HTMLtext += "   <option value=\"eq\">"+multilanguage.getMessage("com.equal")+"</option>";
				}
				if (valuesFromRequest[FIRST_OPERATION].equals("min")){
					HTMLtext += "   <option value=\"min\" selected>"+multilanguage.getMessage("com.minor")+"</option>";
				} else {
					HTMLtext += "   <option value=\"min\">"+multilanguage.getMessage("com.minor")+"</option>";
				}
				if (valuesFromRequest[FIRST_OPERATION].equals("mag")){
					HTMLtext += "   <option value=\"mag\" selected>"+multilanguage.getMessage("com.greater")+"</option>";					
				} else {
					HTMLtext += "   <option value=\"mag\">"+multilanguage.getMessage("com.greater")+"</option>";
				}
			}
			HTMLtext += "</select>";
			HTMLtext += "</td>";
			// FIRST_COMPARISON
			HTMLtext += "<td id=\"r"+row+"c3\">";
			//System.out.println("ab.name="+attributeBean.getName()+" ab.type="+attributeBean.getType());
			if (attributeBean.getType()==AttributeBean.LOOKUP){
				HTMLtext += "<select name=\"r"+row+"d3\">";
				en = lookupVector.elements();
				while (en.hasMoreElements()){
					lb = (LookupBean) en.nextElement();
					if (lb.getId()!=Integer.parseInt((String)valuesFromRequest[FIRST_COMPARISON])){
						HTMLtext += "   <option value=\""+lb.getId()+"\">"+lb.getDescription()+"</option>";
					} else {
						HTMLtext += "   <option value=\""+lb.getId()+"\" selected>"+lb.getDescription()+"</option>";
					}
				}
				HTMLtext += "</select>";
			} else if (attributeBean.getType()==AttributeBean.REFERENCE){
				HTMLtext += "<select name=\"r"+row+"d3\">";
				if (referenceHash.containsKey(attributeBean.getReference())){
					vett = (Vector) referenceHash.get(attributeBean.getReference());
				} 
				if (vett!=null){
					en = vett.elements();
					while (en.hasMoreElements()){
						lb = (LookupBean) en.nextElement();
						if (lb.getId()!=Integer.parseInt((String)valuesFromRequest[FIRST_COMPARISON])){
							HTMLtext += "   <option value=\""+lb.getId()+"\">"+lb.getDescription()+"</option>";
						} else {
							HTMLtext += "   <option value=\""+lb.getId()+"\" selected>"+lb.getDescription()+"</option>";
						}
					}
				}
				HTMLtext += "</select>";
			} else { 
				HTMLtext += "<input type=\"text\" name=\"r"+row+"d3\" value=\""+valuesFromRequest[FIRST_COMPARISON]+"\">";
			}
			HTMLtext += "</td>";
		}
		// AND_OR
		if (valuesFromRequest[AND_OR].length()==0 || valuesFromRequest[AND_OR].equals("AND")){
			// AND
			HTMLtext += "<td id=\"r"+row+"c4\">";
			HTMLtext += "<select name=\"r"+row+"d4\">";
			HTMLtext += "  <option value=\"AND\" selected>AND</option>";
			HTMLtext += "  <option value=\"OR\">OR</option>";
			HTMLtext += "</select>";
			HTMLtext += "</td>";
		} else {
			// OR
			HTMLtext += "<td id=\"r"+row+"c4\">";
			HTMLtext += "<select name=\"r"+row+"d4\">";
			HTMLtext += "  <option value=\"AND\">AND</option>";
			HTMLtext += "  <option value=\"OR\" selected>OR</option>";
			HTMLtext += "</select>";
			HTMLtext += "</td>";
		}
		if (!fullValid){
			if (halfValid){
				// LAST_OPERATION
				HTMLtext += "<td id=\"r"+row+"c5\">";
				HTMLtext += "<select size=\"1\" name=\"r"+row+"d5\">";
				if (attributeBean.getType()==AttributeBean.STRING){
					if (valuesFromRequest[LAST_OPERATION].equals("eq")){
						HTMLtext += "   <option value=\"eq\" selected>"+multilanguage.getMessage("com.equal")+"</option>";
					} else {
						HTMLtext += "   <option value=\"eq\">"+multilanguage.getMessage("com.equal")+"</option>";
					}
					if (valuesFromRequest[LAST_OPERATION].equals("con")){
						HTMLtext += "   <option value=\"con\" selected>"+multilanguage.getMessage("com.contains")+"</option>";
					} else {
						HTMLtext += "   <option value=\"con\">"+multilanguage.getMessage("com.contains")+"</option>";
					}
					if (valuesFromRequest[LAST_OPERATION].equals("sta")){
						HTMLtext += "   <option value=\"sta\" selected>"+multilanguage.getMessage("com.startswith")+"</option>";
					} else {
						HTMLtext += "   <option value=\"sta\">"+multilanguage.getMessage("com.startswith")+"</option>";
					}
				} else {
					if (valuesFromRequest[LAST_OPERATION].equals("eq")){
						HTMLtext += "   <option value=\"eq\" selected>"+multilanguage.getMessage("com.equal")+"</option>";	
					} else {
						HTMLtext += "   <option value=\"eq\">"+multilanguage.getMessage("com.equal")+"</option>";
					}
					if (valuesFromRequest[LAST_OPERATION].equals("min")){
						HTMLtext += "   <option value=\"min\" selected>"+multilanguage.getMessage("com.minor")+"</option>";
					} else {
						HTMLtext += "   <option value=\"min\">"+multilanguage.getMessage("com.minor")+"</option>";
					}
					if (valuesFromRequest[LAST_OPERATION].equals("mag")){
						HTMLtext += "   <option value=\"mag\" selected>"+multilanguage.getMessage("com.greater")+"</option>";					
					} else {
						HTMLtext += "   <option value=\"mag\">"+multilanguage.getMessage("com.greater")+"</option>";
					}
				}
				HTMLtext += "</select>";
				HTMLtext += "</td>";
				// LAST_COMPARISON
				HTMLtext += "<td id=\"r"+row+"c6\">";
				if (attributeBean.getType()==AttributeBean.LOOKUP){
					HTMLtext += "<select name=\"r"+row+"d6\">";
					en = lookupVector.elements();
					while (en.hasMoreElements()){
						lb = (LookupBean) en.nextElement();
						if (lb.getId()!=Integer.parseInt((String)valuesFromRequest[LAST_COMPARISON])){
							HTMLtext += "   <option value=\""+lb.getId()+"\">"+lb.getDescription()+"</option>";
						} else {
							HTMLtext += "   <option value=\""+lb.getId()+"\" selected>"+lb.getDescription()+"</option>";
						}
					}
					HTMLtext += "</select>";
				} else if (attributeBean.getType()==AttributeBean.REFERENCE){
					HTMLtext += "<select name=\"r"+row+"d6\">";
					if (referenceHash.containsKey(attributeBean.getReference())){
						vett = (Vector) referenceHash.get(attributeBean.getReference());
					} 
					if (vett!=null){
						en = vett.elements();
						while (en.hasMoreElements()){
							lb = (LookupBean) en.nextElement();
							if (lb.getId()!=Integer.parseInt((String)valuesFromRequest[LAST_COMPARISON])){
								HTMLtext += "   <option value=\""+lb.getId()+"\">"+lb.getDescription()+"</option>";
							} else {
								HTMLtext += "   <option value=\""+lb.getId()+"\" selected>"+lb.getDescription()+"</option>";
							}
						}
					}
					HTMLtext += "</select>";
				} else { 
					HTMLtext += "<input type=\"text\" name=\"r"+row+"d6\" value=\""+valuesFromRequest[LAST_COMPARISON]+"\">";
				}
				HTMLtext += "</td>";
			} else {
				// LAST_OPERATION
				HTMLtext += "</td>";
				HTMLtext += "<td id=\"r"+row+"c5\">";
				HTMLtext += "<select size=\"1\" name=\"r"+row+"d5\">";
				HTMLtext += "   <option value=\"\"></option>";
				HTMLtext += "</select>";
				HTMLtext += "</td>";
				// LAST_COMPARISON
				HTMLtext += "<td id=\"r"+row+"c6\">";
				HTMLtext += "<select size=\"1\" name=\"r"+row+"d6\">";
				HTMLtext += "   <option value=\"\"></option>";
				HTMLtext += "</select>";
				HTMLtext += "</td>";
			}
		} else {
			// LAST_OPERATION
			HTMLtext += "<td id=\"r"+row+"c5\">";
			HTMLtext += "<select size=\"1\" name=\"r"+row+"d5\">";
			if (attributeBean.getType()==AttributeBean.STRING){
				if (valuesFromRequest[LAST_OPERATION].equals("eq")){
					HTMLtext += "   <option value=\"eq\" selected>"+multilanguage.getMessage("com.equal")+"</option>";
				} else {
					HTMLtext += "   <option value=\"eq\">"+multilanguage.getMessage("com.equal")+"</option>";
				}
				if (valuesFromRequest[LAST_OPERATION].equals("con")){
					HTMLtext += "   <option value=\"con\" selected>"+multilanguage.getMessage("com.contains")+"</option>";
				} else {
					HTMLtext += "   <option value=\"con\">"+multilanguage.getMessage("com.contains")+"</option>";
				}
				if (valuesFromRequest[LAST_OPERATION].equals("sta")){
					HTMLtext += "   <option value=\"sta\" selected>"+multilanguage.getMessage("com.startswith")+"</option>";
				} else {
					HTMLtext += "   <option value=\"sta\">"+multilanguage.getMessage("com.startswith")+"</option>";
				}
			} else {
				if (valuesFromRequest[LAST_OPERATION].equals("eq")){
					HTMLtext += "   <option value=\"eq\" selected>"+multilanguage.getMessage("com.equal")+"</option>";	
				} else {
					HTMLtext += "   <option value=\"eq\">"+multilanguage.getMessage("com.equal")+"</option>";
				}
				if (valuesFromRequest[LAST_OPERATION].equals("min")){
					HTMLtext += "   <option value=\"min\" selected>"+multilanguage.getMessage("com.minor")+"</option>";
				} else {
					HTMLtext += "   <option value=\"min\">"+multilanguage.getMessage("com.minor")+"</option>";
				}
				if (valuesFromRequest[LAST_OPERATION].equals("mag")){
					HTMLtext += "   <option value=\"mag\" selected>"+multilanguage.getMessage("com.greater")+"</option>";					
				} else {
					HTMLtext += "   <option value=\"mag\">"+multilanguage.getMessage("com.greater")+"</option>";
				}
			}
			HTMLtext += "</select>";
			HTMLtext += "</td>";
			// LAST_COMPARISON
			HTMLtext += "<td id=\"r"+row+"c6\">";
			if (attributeBean.getType()==AttributeBean.LOOKUP){
				HTMLtext += "<select name=\"r"+row+"d6\">";
				en = lookupVector.elements();
				while (en.hasMoreElements()){
					lb = (LookupBean) en.nextElement();
					if (lb.getId()!=Integer.parseInt((String)valuesFromRequest[LAST_COMPARISON])){
						HTMLtext += "   <option value=\""+lb.getId()+"\">"+lb.getDescription()+"</option>";
					} else {
						HTMLtext += "   <option value=\""+lb.getId()+"\" selected>"+lb.getDescription()+"</option>";
					}
				}
				HTMLtext += "</select>";
			} else if (attributeBean.getType()==AttributeBean.REFERENCE){
				HTMLtext += "<select name=\"r"+row+"d6\">";
				if (referenceHash.containsKey(attributeBean.getReference())){
					vett = (Vector) referenceHash.get(attributeBean.getReference());
				} 
				if (vett!=null){
					en = vett.elements();
					while (en.hasMoreElements()){
						lb = (LookupBean) en.nextElement();
						if (lb.getId()!=Integer.parseInt((String)valuesFromRequest[LAST_COMPARISON])){
							HTMLtext += "   <option value=\""+lb.getId()+"\">"+lb.getDescription()+"</option>";
						} else {
							HTMLtext += "   <option value=\""+lb.getId()+"\" selected>"+lb.getDescription()+"</option>";
						}
					}
				}
				HTMLtext += "</select>";
			} else { 
				HTMLtext += "<input type=\"text\" name=\"r"+row+"d6\" value=\""+valuesFromRequest[LAST_COMPARISON]+"\">";
			}
			HTMLtext += "</td>";
		}
		HTMLtext += "<td>";
		// beacause Vector start form 0
		HTMLtext += "<a href=\"javascript:createReportForm.mpath.value='"+row+"';createReportForm.method.value='removefilter';createReportForm.submit();\"><img class=\"icon\" src=\"images/ico_del.gif\" alt=\"del\" title=\"del\" class=\"icon\"></a>";
		HTMLtext += "</td>";
		HTMLtext += "</tr>\n";
		return HTMLtext;
	}
	
	public int getSubPage() {
		return subPage;
	}
	public void setSubPage(int subPage) {
		this.subPage = subPage;
	}
	public int getRow() {
		return row;
	}
	public void setRow(int row) {
		this.row = row;
	}
	public boolean isFullValid() {
		return fullValid;
	}
	public boolean isHalfValid() {
		return halfValid;
	}
	public AttributeBean getAttributeBean() {
		return attributeBean;
	}
	public void setAttributeBean(AttributeBean attributeBean) {
		this.attributeBean = attributeBean;
	}
	public String getAlias() {
		return alias;
	}
	public void setAlias(String alias) {
		this.alias = alias;
	}
	public Vector getAttributeVector() {
		return attributeVector;
	}
	public void setAttributeVector(Vector attributeVector) {
		this.attributeVector = attributeVector;
	}
	public void setLookupVector(Vector lookupVector) {
		this.lookupVector = lookupVector;
	}
	public Hashtable getReferenceHash() {
		return referenceHash;
	}
	public void setReferenceHash(Hashtable referenceHash) {
		this.referenceHash = referenceHash;
	}
	
} // end of class
