package cmdbuild.wizards.reportutils;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

import cmdbuild.collection.AttributeBean;
import cmdbuild.collection.ClassBean;
import cmdbuild.collection.ReportMenuBean;
import cmdbuild.utils.QueryTokenizer;

public class ReportQueryFactory implements Serializable{
	
	static final long serialVersionUID = 1L;
	private String startClass;
	private Vector pathVector;  // Vector of Vectors of ReportMenuBean 
	private Vector mPathVector; // String (mpath) vector  
	private String andOr;
	private Vector pathTitles; // titles for page 3,   base path,path1,path2 etc...
	private Hashtable involvedClassesForPath; // contains vectors of string
	private int aliasCount; // key=class value=total number of alias for this class in the query
	private Hashtable aliasForPage; // key=subpage  value=vector of String
	private Vector aliasOrders;  // list of AttributeBean
	private Hashtable filtersForPage;
	private boolean horizontalPrint; 
	private int cCount; // used to count class alias
	private int dCount; // used to count domain alias
	private Hashtable attributesToPrintHash;
	
	public ReportQueryFactory() {
		super();
		pathVector = new Vector();
		mPathVector = new Vector();
		aliasForPage = new Hashtable();
		pathTitles = new Vector();
		aliasOrders = new Vector();
		filtersForPage = new Hashtable();
		andOr = "AND";
		aliasCount = 0;
		cCount = 1; // because select 0 is used manually
		dCount = 0; //
		attributesToPrintHash = new Hashtable();
	}
	
	public void showStatistics(){
		System.out.println("=====Start statistics=======");
		System.out.println("startClass="+startClass);
		System.out.println("pathVector="+pathVector.size()); 
		System.out.println("mPathVector="+mPathVector.size());  
		System.out.println("andOr="+andOr);
		System.out.println("pathTitles="+pathTitles.size());
		System.out.println("involvedClassesForPath="+involvedClassesForPath.size());
		System.out.println("aliasCount="+aliasCount); 
		System.out.println("aliasForPage="+aliasForPage.size());
		System.out.println("aliasOrders="+aliasOrders.size());
		System.out.println("filtersForPage="+filtersForPage.size());
		System.out.println("horizontalPrint="+horizontalPrint); 
		System.out.println("cCount="+cCount);
		System.out.println("dCount="+dCount);
		System.out.println("=======End statistics=======");
	}
	
	/*
	 *  PathVectors
	 */
	public int countPaths(){
		return pathVector.size();
	}
	public void clearPaths(){
		pathVector = new Vector();
		mPathVector = new Vector();
		pathTitles = new Vector();
	}
	public void addPaths(Vector path, String mpath){
		pathVector.add(path);
		mPathVector.add(mpath);
		//System.out.println("addPaths actual size="+pathVector.size()+" mapth size="+mPathVector.size());
	}
	public void removePaths(int pos){
		pathVector.removeElementAt(pos);
		mPathVector.removeElementAt(pos);
		pathTitles.removeElementAt(pos);
	}
	/*
	 *  Titles
	 */
	public void setTitle(String title, int pos){
		if (pos>pathTitles.size()-1){
			pathTitles.add(title);
		} else {
			pathTitles.setElementAt(title,pos);
		}
	}
	public String getTitle(int pos){
		return (String) pathTitles.elementAt(pos);
	}
	/*
	 * Involved Classes
	 */	
	public void updateInvolvedClasses(String name){
		involvedClassesForPath = new Hashtable();
		Vector vettp = new Vector();
		vettp.add(name);
		involvedClassesForPath.put("0",vettp); // base path
		//System.out.println("updateInvolvedClasses path0 ="+vettp.size());
		Enumeration en = pathVector.elements();
		Enumeration eni = null;
		ReportMenuBean rmb = null;
		Vector vett = null;
		int p = 1;
		while (en.hasMoreElements()){
			vett = (Vector) en.nextElement();
			eni = vett.elements();
			vettp = new Vector();
			while (eni.hasMoreElements()){
				rmb = (ReportMenuBean) eni.nextElement();
				if (rmb.isDirect()){
					name = rmb.getClass2(); 
				} else {
					name = rmb.getClass1();
				}
				vettp.add(name);
			}
			involvedClassesForPath.put(p+"",vettp);
			//System.out.println("updateInvolvedClasses path"+p+" ="+vettp.size());
			p++;
		}
	}
	public Vector getInvolvedClassesForPage(int subPage){
		Vector res = (Vector) involvedClassesForPath.get(subPage+"");
		//System.out.println("getInvolvedClasses path"+subPage+" ="+res.size());
		return res;
	}
	public Vector getInvolvedClasses(){
		Vector vett = null;
		HashSet h = new HashSet();
		Enumeration en = involvedClassesForPath.elements();
		while (en.hasMoreElements()){
			vett = (Vector) en.nextElement();
			h.addAll(vett);
		}
		return new Vector(h);
	}
	/*
	 * Filters 
	 */
	public void clearFilters(){
		filtersForPage = new Hashtable();
	} 
	public void addFilterForPage(ReportFilter rf, int subPage){
		Vector vett = null;
		if (!filtersForPage.containsKey(subPage+"")){
			vett = new Vector();
			vett.add(rf);
			filtersForPage.put(subPage+"",vett);
			//System.out.println("addFilter create vett for page "+subPage+" size="+vett.size());
		} else {
			vett = (Vector) filtersForPage.get(subPage+"");
			//System.out.println("addFilter get vett for page "+subPage+" size="+vett.size());
			vett.add(rf);
			filtersForPage.put(subPage+"",vett);
		}
		//System.out.println("addFilter for page "+subPage);
	} 
	public void removeFilterAtPage(int subPage, int pos){
		Vector vett = (Vector) filtersForPage.get(subPage+"");
		//System.out.println("removing filter "+pos+" in page "+subPage);
		vett.removeElementAt(pos);
		Enumeration en = vett.elements();
		ReportFilter rf = null;
		int n = 0;
		vett = new Vector();
		while (en.hasMoreElements()){
			rf = (ReportFilter) en.nextElement();
			rf.setRow(n);
			n++;
		}
	} 
	public Vector getFiltersForPage(int subPage){
		Vector vett = new Vector();
		if (filtersForPage.containsKey(subPage+"")){
			vett = (Vector) filtersForPage.get(subPage+"");
		}
		return vett;
	}
	// return path 0 (always added)+ path requested
	public Vector getFiltersPlus(int subPage){
		subPage++;
		Vector vett = new Vector();
		Vector v0 = new Vector();
		Vector vp = new Vector();
		if (filtersForPage.containsKey("0")){
			v0 = (Vector) filtersForPage.get("0");
		}
		//System.out.println("getFilterPlus page0 size="+v0.size());
		if (filtersForPage.containsKey(subPage+"")){
			vp = (Vector) filtersForPage.get(subPage+"");
		}
		//System.out.println("getFilterPlus page"+subPage+" size="+vp.size());
		vett.addAll(v0);
		vett.addAll(vp);
		//System.out.println("getFilterPlus page0 + page"+subPage+" size="+vett.size());
		return vett;
	}
	/*
	 *  Alias
	 */
	public void removeSelectForPage(int subPage){
		if (aliasForPage.containsKey(subPage+"")){
			Vector vett = new Vector();
			aliasForPage.put(subPage+"",vett);
		} 
	}
	public void addSelectForPage(String name, int subPage){
		ClassBean cb = null;
		Vector vett = null;
		Vector vettc = null;
		aliasCount++;
		int count = 0;
		if (!aliasForPage.containsKey(subPage+"")){
			vett = new Vector();
			cb = new ClassBean();
			cb.setName(name);
			// count previous items
			for (int i=0;i<subPage;i++){
				vettc = (Vector) aliasForPage.get(i+"");
				count += vettc.size();
			}
			cb.setComment("select"+count);
			vett.add(cb);
			System.out.println("addSelectForPage name="+name+" subPage="+subPage+" select="+cb.getComment());
			aliasForPage.put(subPage+"",vett);
		} else {
			vett = (Vector) aliasForPage.get(subPage+"");
			count = vett.size();
			cb = new ClassBean();
			cb.setName(name);
			// count previous items
			for (int i=0;i<subPage;i++){
				vettc = (Vector) aliasForPage.get(i+"");
				count += vettc.size();
			}
			cb.setComment("select"+count);
			vett.add(cb);
			System.out.println("addSelectForPage name="+name+" subPage="+subPage+" select="+cb.getComment());
			aliasForPage.put(subPage+"",vett);
		}
	}
	public Vector getSelectForPage(int subPage){
		Vector vett = new Vector();
		if (aliasForPage.containsKey(subPage+"")){
			vett = (Vector) aliasForPage.get(subPage+"");
		}
		return vett;
	}
	
	/*
	 * attributesToPrint
	 */
	
	public Vector getAttributesToPrint(String selectName){
		Vector vett = new Vector();
		if (attributesToPrintHash.containsKey(selectName)){
			vett = (Vector) attributesToPrintHash.get(selectName);
		}
		return vett;
	}
	
	public void setAttributesToPrint(String selectName, Vector attributes, int path){
		if (attributes!=null){
			Enumeration en = attributes.elements();
			AttributeBean ab = null;
			AttributeBean abc = null;
			// saving subpage
			Vector vett = new Vector();
			while (en.hasMoreElements()){
				ab = (AttributeBean) en.nextElement();
				abc = AttributeBean.getCopyOf(ab);
				abc.setIndex(path);
				abc.setBaseDSP(""); // used for text alinment (left,right,center,justify)
				vett.add(abc);
			}
			System.out.println("Adding for "+selectName+" "+vett.size()+" elements");
			attributesToPrintHash.put(selectName,vett);
		}
	}
	
	public boolean isAttributesToPrint(String selectName, AttributeBean bean){
		boolean result = false;
		Vector vett = new Vector();
		if (attributesToPrintHash.containsKey(selectName)){
			vett = (Vector) attributesToPrintHash.get(selectName);
		}
		AttributeBean ab = null;
		Enumeration en = vett.elements();
		while (en.hasMoreElements()){
			ab = (AttributeBean) en.nextElement();
			if (bean.getName().equals(ab.getName())){
				result = true;
				break;
			}
		}
		return result;
	}

	public void createOrders(){
		System.out.println("createOrders()");
		int count = attributesToPrintHash.size();
		Vector vett = null;
		String select = "select";
		aliasOrders = new Vector();
		for (int i=0;i<count;i++){
			if (attributesToPrintHash.containsKey(select+i)){
				vett = (Vector) attributesToPrintHash.get(select+i);
				System.out.println("adding select="+select+i+" size="+vett.size());
				aliasOrders.addAll(vett);
			}
		}
	}
	
	/*
	public void createOrders(){
		System.out.println("createOrders()");
		Enumeration en = attributesToPrintHash.keys();
		Vector vett = null;
		String select = "";
		aliasOrders = new Vector();
		while (en.hasMoreElements()){
			select = (String) en.nextElement();
			System.out.println("select="+select);
			vett = (Vector) attributesToPrintHash.get(select);
			System.out.println("adding select="+select+" size="+vett.size());
			aliasOrders.addAll(vett);
		}
	}
	*/

	
	/*
	 *  Orders
	 */
	
	public AttributeBean getOrderAt(int pos){
		return (AttributeBean)aliasOrders.elementAt(pos);
	}
	// position++
	public void downOrderAt(int pos){
		AttributeBean ab1 = null;
		AttributeBean ab2 = null;
		if (pos<aliasOrders.size()-1){
			ab1 = (AttributeBean) aliasOrders.elementAt(pos);
			ab2 = (AttributeBean) aliasOrders.elementAt(pos+1);
			aliasOrders.setElementAt(ab1,pos+1);
			aliasOrders.setElementAt(ab2,pos);
		}
	}
	// position --
	public void upOrderAt(int pos){
		AttributeBean ab1 = null;
		AttributeBean ab2 = null;
		if (pos>0){
			ab1 = (AttributeBean) aliasOrders.elementAt(pos);
			ab2 = (AttributeBean) aliasOrders.elementAt(pos-1);
			aliasOrders.setElementAt(ab1,pos-1);
			aliasOrders.setElementAt(ab2,pos);
		}
	}
	// position last
	public void lowerOrderAt(int pos){
		AttributeBean ab = null;
		AttributeBean target = null;
		Vector temp = new Vector();
		for (int i=0;i<aliasOrders.size();i++){
			ab = (AttributeBean) aliasOrders.elementAt(i);
			if (i!=pos){
				temp.add(ab);
			} else {
				target = ab;
			}
		}
		temp.add(target);
		aliasOrders = temp;
	}
	// position first
	public void upperOrderAt(int pos){
		AttributeBean ab = null;
		AttributeBean target = null;
		Vector temp = new Vector();
		for (int i=0;i<aliasOrders.size();i++){
			ab = (AttributeBean) aliasOrders.elementAt(i);
			if (i!=pos){
				temp.add(ab);
			} else {
				target = ab;
			}
		}
		aliasOrders = new Vector();
		aliasOrders.add(target);
		aliasOrders.addAll(temp);
	}
	// return path 0 (always added)+ path requested
	public Vector getOrdersPlus(int path){
		path++;
		Vector result = new Vector();
		Enumeration en = aliasOrders.elements();
		AttributeBean ab = null;
		while (en.hasMoreElements()){
			ab = (AttributeBean) en.nextElement();
			//System.out.println("getOrderPlus"+ab.getClassBean().getName()+"-"+ab.getName()+" index="+ab.getIndex()+"=0 || ="+path+"?");
			if (ab.getIndex()==0 || ab.getIndex()==path){
				result.add(ab);
				//System.out.println("adding ab");
			}
		}
		return result;
	}
	public Vector getOrders(){
		return aliasOrders;
	}
	
	public void initializeCouters(){
		cCount = 1; // because select 0 is used manually
		dCount = 0; //
	}

	public String makeQueryForPath(int path){
		System.out.println("MAKE QUERY FOR PATH="+path);
		String query = "";
		String sel = ""; // SELECT
		String fro = ""; // FROM
		String whe = ""; // WHERE
		String ord = ""; // ORDER BY
		String str = "";
		Enumeration en = pathVector.elements();
		System.out.println("pathVector.size="+pathVector.size());
		ReportMenuBean rmb = null;
		Vector vett = null;
		Enumeration eni = null;
		if (pathVector.size()>0){
			vett = (Vector) pathVector.elementAt(path);
		} else {
			vett = new Vector();
		}
		System.out.println("pathVector.elements() leftvett.size="+vett.size());
		eni = vett.elements();
		// skipping first element
		if (eni.hasMoreElements()){
			eni.nextElement();
		}
		while (eni.hasMoreElements()){
			rmb = (ReportMenuBean) eni.nextElement();
			if (fro.length()==0){
				if (rmb.isDirect()){
					fro = "SELECT * FROM \""+rmb.getClass1()+"\" AS select0";
					fro +=" LEFT OUTER JOIN \"Map_"+rmb.getDomain()+"\" AS dom"+dCount;
					fro +=" ON select0.\"IdClass\"=dom"+dCount+".\"IdClass1\" AND select0.\"Id\"=dom"+dCount+".\"IdObj1\" AND select0.\"Status\"='A' AND dom"+dCount+".\"Status\"='A'";
					fro +=" LEFT OUTER JOIN \""+rmb.getClass2()+"\" AS select"+cCount;
					fro +=" ON select"+cCount+".\"IdClass\"=dom"+dCount+".\"IdClass2\" AND select"+cCount+".\"Id\"=dom"+dCount+".\"IdObj2\" AND select"+cCount+".\"Status\"='A' AND dom"+dCount+".\"Status\"='A'";
					cCount++;
					dCount++;
				} else {
					fro = "SELECT * FROM \""+rmb.getClass2()+"\" AS select0";
					fro +=" LEFT OUTER JOIN \"Map_"+rmb.getDomain()+"\" AS dom"+dCount;
					fro +=" ON select0.\"IdClass\"=dom"+dCount+".\"IdClass2\" AND select0.\"Id\"=dom"+dCount+".\"IdObj2\" AND select0.\"Status\"='A' AND dom"+dCount+".\"Status\"='A'";
					fro +=" LEFT OUTER JOIN \""+rmb.getClass1()+"\" AS select"+cCount;
					fro +=" ON select"+cCount+".\"IdClass\"=dom"+dCount+".\"IdClass1\" AND select"+cCount+".\"Id\"=dom"+dCount+".\"IdObj1\" AND select"+cCount+".\"Status\"='A' AND dom"+dCount+".\"Status\"='A'";
					cCount++;
					dCount++;
				}
			} else {
				if (rmb.isDirect()){
					fro +=" LEFT OUTER JOIN \"Map_"+rmb.getDomain()+"\" AS dom"+dCount;
					fro +=" ON select"+(cCount-1)+".\"IdClass\"=dom"+dCount+".\"IdClass1\" AND select"+(cCount-1)+".\"Id\"=dom"+dCount+".\"IdObj1\" AND select"+(cCount-1)+".\"Status\"='A' AND dom"+dCount+".\"Status\"='A'";
					fro +=" LEFT OUTER JOIN \""+rmb.getClass2()+"\" AS select"+cCount;
					fro +=" ON select"+cCount+".\"IdClass\"=dom"+dCount+".\"IdClass2\" AND select"+cCount+".\"Id\"=dom"+dCount+".\"IdObj2\" AND select"+cCount+".\"Status\"='A' AND dom"+dCount+".\"Status\"='A'";
					cCount++;
					dCount++;
				} else {
					fro +=" LEFT OUTER JOIN \"Map_"+rmb.getDomain()+"\" AS dom"+dCount;
					fro +=" ON select"+(cCount-1)+".\"IdClass\"=dom"+dCount+".\"IdClass2\" AND select"+(cCount-1)+".\"Id\"=dom"+dCount+".\"IdObj2\" AND select"+(cCount-1)+".\"Status\"='A' AND dom"+dCount+".\"Status\"='A'";
					fro +=" LEFT OUTER JOIN \""+rmb.getClass1()+"\" AS select"+cCount;
					fro +=" ON select"+cCount+".\"IdClass\"=dom"+dCount+".\"IdClass1\" AND select"+cCount+".\"Id\"=dom"+dCount+".\"IdObj1\" AND select"+cCount+".\"Status\"='A' AND dom"+dCount+".\"Status\"='A'";
					cCount++;
					dCount++;
				}
			}
		}
		// minimal query
		if (fro.length()==0){
			fro = "SELECT * FROM \""+startClass+"\" AS select0";
		}
		// creating select
		vett = getOrdersPlus(path);
		System.out.println("getOrdersPlus selectvett.size="+vett.size());
		en = vett.elements();
		AttributeBean ab = null;
		while (en.hasMoreElements()){
			ab = (AttributeBean) en.nextElement();
			if (sel.length()==0){
				str = ab.getName().toLowerCase();
				sel = "SELECT "+ab.getClassBean().getComment()+".\""+ab.getName()+"\" AS "+ab.getClassBean().getComment()+str;
			} else {
				str = ab.getName().toLowerCase();
				sel += ", "+ab.getClassBean().getComment()+".\""+ab.getName()+"\" AS "+ab.getClassBean().getComment()+str;
			}
		}
		// creating WHERE 
		vett = getFiltersPlus(path);
		System.out.println("getOrdersPlus wherevett.size="+vett.size());
		en = vett.elements();
		ReportFilter rf = null;			
		while (en.hasMoreElements()){
			rf = (ReportFilter) en.nextElement();
			System.out.println("WHERE rf from subPage="+rf.getSubPage());
			if (whe.length()==0 && rf.toQuery().length()>0){
				whe = "WHERE "+rf.toQuery();
			} else if (rf.toQuery().length()>0){
				whe += " AND "+rf.toQuery() ;
			}
		}
		// adding 
		if (whe.length()==0){
			whe = "WHERE select0.\"Status\"='A' ";
		} else if (rf.toQuery().length()>0){
			whe += " AND select0.\"Status\"='A' ";
		}
		// trunc fro
		fro = fro.substring(fro.indexOf("FROM"),fro.length());
		// creating ORDER BY 
		vett = getOrdersPlus(path);
		System.out.println("getOrdersPlus vett.size="+vett.size());
		en = vett.elements();
		while (en.hasMoreElements()){
			ab = (AttributeBean) en.nextElement();
			if (ord.length()==0){
				ord = "ORDER BY "+ab.getClassBean().getComment()+".\""+ab.getName()+"\"";
			} else {
				ord += ", "+ab.getClassBean().getComment()+".\""+ab.getName()+"\"";
			}
		}
		// adding order
		System.out.println("sel="+sel+"* fro="+fro+"* whe="+whe+"*"+ord+"*");
		query = sel+" "+fro+" "+whe+" "+ord;
		OptimizeQuery oq = new OptimizeQuery(query);
		query = oq.getFastQuery();
		return query;
	}
	public String makeQueryAndOr(Vector querys){
		String query = "";
		String sel = "";
		HashSet ssel = new HashSet();
		String fro = "";
		String joi = "";
		Vector joinV = new Vector();
		String whe = "";
		HashSet swhe = new HashSet();
		String ord = "";
		HashSet sord = new HashSet();
		String line = "";
		String str = "";
		StringTokenizer tok = null;
		if (andOr.equals("AND")){
			for (int i=0;i<querys.size();i++){
				query = (String) querys.elementAt(i);
				// SELECT
				line = query.substring(7,query.indexOf("FROM"));
				//System.out.println("?select line="+line+"*");
				tok = new StringTokenizer(line,",",false);
				while (tok.hasMoreElements()){
					str = tok.nextToken();
					str = str.trim();
					//System.out.println("select token="+str+"*");
					ssel.add(str);
				}
				// FROM 
				if (fro.length()==0){
					if (query.indexOf("LEFT")!=-1){
						fro = query.substring(query.indexOf("FROM"),query.indexOf("LEFT"));	
					} else if(query.indexOf("WHERE")!=-1){
						fro = query.substring(query.indexOf("FROM"),query.indexOf("WHERE"));
					} else {
						fro = query.substring(query.indexOf("FROM"),query.indexOf("ORDER"));
					}
					//System.out.println("from="+fro+"*");
				}
				// JOIN
				if (query.indexOf("LEFT")!=-1){
					if (query.indexOf("WHERE")!=-1){
						line = query.substring(query.indexOf("LEFT"),query.indexOf("WHERE"));
					} else {
						line = query.substring(query.indexOf("LEFT"),query.indexOf("ORDER"));
					}
				} else {
					line = "";
				}
				//System.out.println("?join line="+line);
				QueryTokenizer qt = new QueryTokenizer(line,"LEFT",true);
				Vector vett = qt.getTokens();
				joinV.addAll(vett);
				// WHERE
				if (query.indexOf("WHERE")!=-1){
					line = query.substring(query.indexOf("WHERE")+6,query.indexOf("ORDER"));
				} else {
					line = "";
				}
				//System.out.println("?where line="+line);
				qt = new QueryTokenizer(line,"AND",false);
				vett = qt.getTokens();
				Enumeration en = vett.elements();
				while (en.hasMoreElements()){
					str = (String) en.nextElement();
					//System.out.println("where token="+str+"*");
					swhe.add(str);
				}
				// ORDER BY
				line = query.substring(query.indexOf("ORDER")+9,query.length());
				//System.out.println("?order line="+line);
				tok = new StringTokenizer(line,",",false);
				while (tok.hasMoreElements()){
					str = tok.nextToken();
					str = str.trim();
					//System.out.println("order token="+str+"*");
					sord.add(str);
				}
			}
			// Regenerating query
			// SELECT
			Iterator it = ssel.iterator();			
			while (it.hasNext()){
				str = (String) it.next();
				if (sel.length()>0){
					sel += ", "+str;
				} else {
					sel = "SELECT "+str;
				}			
			}
			//System.out.println("sel="+sel);
			//System.out.println("fro="+fro);
			// JOIN
			Enumeration en = joinV.elements();
			while (en.hasMoreElements()){
				str = (String) en.nextElement();
				if (joi.length()>0){
					joi += " "+str;
				} else {
					joi = str;
				}
			}
			//System.out.println("joi="+joi);
			// WHERE
			it = swhe.iterator();			
			while (it.hasNext()){
				str = (String) it.next();
				if (whe.length()>0){
					whe += " AND "+str;
				} else {
					whe = "WHERE "+str;
				}			
			}
			//System.out.println("whe="+whe);
			// adding ORDER BY 
			Vector vett = getOrders();
			en = vett.elements();
			AttributeBean ab = null;
			while (en.hasMoreElements()){
				ab = (AttributeBean) en.nextElement();
				str = ab.getClassBean().getComment()+ab.getName();
				str = str.toLowerCase();
				if (ord.length()>0){
					ord += ", "+str;
				} else {
					ord = "ORDER BY "+str;
				}
			}
			//System.out.println("ord="+ord);
			query = sel+" "+fro+" "+joi+" "+whe+" "+ord;
			//System.out.println("FINAL QUERY:");
			//System.out.println(query);
		} else { // OR
			// vector of hashtables for every query
			Vector selectV = new Vector();
			Hashtable global = new Hashtable();
			Hashtable temp = null;
			String alias = "";
			for (int i=0;i<querys.size();i++){
				query = (String) querys.elementAt(i);
				// SELECT
				line = query.substring(7,query.indexOf("FROM"));
				//System.out.println("?select line="+line);
				tok = new StringTokenizer(line,",",false);
				ssel = new HashSet();
				while (tok.hasMoreElements()){
					str = tok.nextToken();
					str = str.trim();
					//System.out.println("select token="+str+"*");
					ssel.add(str);
				}
				Iterator it = ssel.iterator();
				temp = new Hashtable();
				// key = select0code
				// value = select0."Code" AS select0code
				while (it.hasNext()){
					str = (String) it.next();
					alias = str.substring(str.indexOf("AS")+3,str.length());
					temp.put(alias,str);
					//System.out.println("saving key="+alias+" value="+str);
					if (!global.containsKey(alias)){
						global.put(alias,str);
						//System.out.println("global key="+alias+" value="+str);
					}
				}
				// saving mapping for this query
				selectV.add(temp);
			}
			String finalQuery = "";
			Enumeration en = null;
			for (int i=0;i<querys.size();i++){
				query = (String) querys.elementAt(i);
				// drop select and drop order by
				query = query.substring(query.indexOf("FROM"),query.indexOf("ORDER"));
				en = global.keys();
				sel = "";
				while (en.hasMoreElements()){
					alias = (String) en.nextElement();
					temp = (Hashtable) selectV.elementAt(i);
					if (temp.containsKey(alias)){
						//System.out.println("query"+i+" ck "+alias);
						if (sel.length()>0){
							sel += ", "+(String) global.get(alias);
						} else {
							sel = "SELECT "+(String) global.get(alias);
						}
					} else {
						//System.out.println("query"+i+" nk "+alias);
						if (sel.length()>0){
							sel += ", '' AS "+alias;
						} else {
							sel = "SELECT '' AS "+alias;
						}
					}
				}
				query = sel+" "+query;
				if (finalQuery.length()>0){
					finalQuery += " UNION "+query;
				} else {
					finalQuery = query;
				}
			}
			// adding ORDER BY 
			Vector vett = getOrders();
			en = vett.elements();
			AttributeBean ab = null;
			while (en.hasMoreElements()){
				ab = (AttributeBean) en.nextElement();
				alias = ab.getClassBean().getComment()+ab.getName();
				alias = alias.toLowerCase();
				if (ord.length()>0){
					ord += ", "+alias;
				} else {
					ord = " ORDER BY "+alias;
				}
			}
			finalQuery += ord;
			//System.out.println("FINAL QUERY:");
			//System.out.println(finalQuery);
			query = finalQuery;
		}
		return query;
	} 
	
	/*
	 * 	Getter and setters
	 */
	public String getAndOr() {
		return andOr;
	}
	public void setAndOr(String andOr) {
		this.andOr = andOr;
	}
	public Vector getPathVector() {
		return pathVector;
	}
	public boolean isHorizontalPrint() {
		return horizontalPrint;
	}
	public void setHorizontalPrint(boolean horizontalPrint) {
		this.horizontalPrint = horizontalPrint;
	}
	public String getStartClass() {
		return startClass;
	}
	public void setStartClass(String startClass) {
		this.startClass = startClass;
	}
	public int getAliasCount() {
		return aliasCount;
	}
	public void setAliasCount(int aliasCount) {
		this.aliasCount = aliasCount;
	}
	
} // end of class
