package it.comune.udine.cmdb.utils;

import java.security.spec.KeySpec;
import java.security.spec.AlgorithmParameterSpec;
import java.security.InvalidAlgorithmParameterException;
import java.security.NoSuchAlgorithmException;
import java.security.InvalidKeyException;
import java.security.spec.InvalidKeySpecException;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.spec.PBEParameterSpec;

public class SecurityEncrypterOriginal {

	Cipher ecipher;
	Cipher dcipher;

	public SecurityEncrypterOriginal() {
		super();
		// TODO Auto-generated constructor stub
	}

	public SecurityEncrypterOriginal(SecretKey key, String algorithm) {
		try {
			ecipher = Cipher.getInstance(algorithm);
	        dcipher = Cipher.getInstance(algorithm);
	        ecipher.init(Cipher.ENCRYPT_MODE, key);
	        dcipher.init(Cipher.DECRYPT_MODE, key);
	    } catch (NoSuchPaddingException e) {
	        System.out.println("EXCEPTION: NoSuchPaddingException");
	    } catch (NoSuchAlgorithmException e) {
	        System.out.println("EXCEPTION: NoSuchAlgorithmException");
	    } catch (InvalidKeyException e) {
	        System.out.println("EXCEPTION: InvalidKeyException");
	    }
	}

	public SecurityEncrypterOriginal(String passPhrase) {

	        // 8-bytes Salt
	        byte[] salt = {
	            (byte)0xA9, (byte)0x9B, (byte)0xC8, (byte)0x32,
	            (byte)0x56, (byte)0x34, (byte)0xE3, (byte)0x03
	        };

	        // Iteration count
	        int iterationCount = 19;

	        try {

	            KeySpec keySpec = new PBEKeySpec(passPhrase.toCharArray(), salt, iterationCount);
	            SecretKey key = SecretKeyFactory.getInstance("PBEWithMD5AndDES").generateSecret(keySpec);

	            ecipher = Cipher.getInstance(key.getAlgorithm());
	            dcipher = Cipher.getInstance(key.getAlgorithm());

	            // Prepare the parameters to the cipthers
	            AlgorithmParameterSpec paramSpec = new PBEParameterSpec(salt, iterationCount);

	            ecipher.init(Cipher.ENCRYPT_MODE, key, paramSpec);
	            dcipher.init(Cipher.DECRYPT_MODE, key, paramSpec);

	        } catch (InvalidAlgorithmParameterException e) {
	            System.out.println("EXCEPTION: InvalidAlgorithmParameterException");
	        } catch (InvalidKeySpecException e) {
	            System.out.println("EXCEPTION: InvalidKeySpecException");
	        } catch (NoSuchPaddingException e) {
	            System.out.println("EXCEPTION: NoSuchPaddingException");
	        } catch (NoSuchAlgorithmException e) {
	            System.out.println("EXCEPTION: NoSuchAlgorithmException");
	        } catch (InvalidKeyException e) {
	            System.out.println("EXCEPTION: InvalidKeyException");
	        }
	    }

	    public String encrypt(String str) {
	        try {
	            // Encode the string into bytes using utf-8
	            byte[] utf8 = str.getBytes("UTF8");

	            // Encrypt
	            byte[] enc = ecipher.doFinal(utf8);

	            // Encode bytes to base64 to get a string
	            return new sun.misc.BASE64Encoder().encode(enc);

	        } catch (Exception e) {
	        }
	        return null;
	    }

	    public String decrypt(String str) {
	        try {
	            // Decode base64 to get bytes
	            byte[] dec = new sun.misc.BASE64Decoder().decodeBuffer(str);
	            // Decrypt
	            byte[] utf8 = dcipher.doFinal(dec);
	            // Decode using utf-8
	            return new String(utf8, "UTF8");
	        } catch (Exception e) {
	        } 
	        return null;
	    }

	    public static void testUsingSecretKey() {
	        try {

	            System.out.println();
	            System.out.println("+----------------------------------------+");
	            System.out.println("|  -- Test Using Secret Key Method --    |");
	            System.out.println("+----------------------------------------+");
	            System.out.println();

	            String secretString = "admin";

	            // Generate a temporary key for this example. In practice, you would
	            // save this key somewhere. Keep in mind that you can also use a 
	            // Pass Phrase.
	            SecretKey desKey       = KeyGenerator.getInstance("DES").generateKey();

	            // Create encrypter/decrypter class
	            SecurityEncrypterOriginal desEncrypter = new SecurityEncrypterOriginal(desKey, desKey.getAlgorithm());
	            String desEncrypted       = desEncrypter.encrypt(secretString);
	            String desDecrypted       = desEncrypter.decrypt(desEncrypted);

	            // Print out values
	            System.out.println(desKey.getAlgorithm() + " Encryption algorithm");
	            System.out.println("    Original String  : " + secretString);
	            System.out.println("    Encrypted String : " + desEncrypted);
	            System.out.println("    Decrypted String : " + desDecrypted);
	            System.out.println();

	        } catch (NoSuchAlgorithmException e) {
	        }
	    }

	    public static void testUsingPassPhrase() {

	        System.out.println();
	        System.out.println("+----------------------------------------+");
	        System.out.println("|  -- Test Using Pass Phrase Method --   |");
	        System.out.println("+----------------------------------------+");
	        System.out.println();

	        String secretString = "admin";
	        String passPhrase   = "27071978";

	        SecurityEncrypterOriginal desEncrypter = new SecurityEncrypterOriginal(passPhrase);
	        String desEncrypted       = desEncrypter.encrypt(secretString);
	        String desDecrypted       = desEncrypter.decrypt(desEncrypted);

	        // Print out values
	        System.out.println("PBEWithMD5AndDES Encryption algorithm");
	        System.out.println("    Original String  : " + secretString);
	        System.out.println("    Encrypted String : " + desEncrypted);
	        System.out.println("    Decrypted String : " + desDecrypted);
	        System.out.println();
	    }

} // end of class
